# -*- coding: utf-8 -*-
"""
Git工具模块
该模块提供了与Git仓库交互的工具。
包含以下功能：
- 查找Git仓库的根目录
- 检查是否有未提交的更改
- 获取两个哈希值之间的提交历史
- 获取最新提交的哈希值
- 从Git差异中提取修改的行范围
"""
import datetime
import os
import re
import subprocess
import sys
from typing import Any, Dict, List, Set, Tuple

from jarvis.jarvis_utils.config import get_data_dir, is_confirm_before_apply_patch
from jarvis.jarvis_utils.output import OutputType, PrettyOutput
from jarvis.jarvis_utils.input import user_confirm


def find_git_root_and_cd(start_dir: str = ".") -> str:
    """
    切换到给定路径的Git根目录，如果不是Git仓库则初始化。

    参数:
        start_dir (str): 起始查找目录，默认为当前目录。

    返回:
        str: Git仓库根目录路径。如果目录不是Git仓库，则会初始化一个新的Git仓库。
    """
    os.chdir(start_dir)
    try:
        git_root = os.popen("git rev-parse --show-toplevel").read().strip()
        if not git_root:
            subprocess.run(["git", "init"], check=True)
            git_root = os.path.abspath(".")
    except subprocess.CalledProcessError:
        # 如果不是Git仓库，初始化一个新的
        subprocess.run(["git", "init"], check=True)
        git_root = os.path.abspath(".")
    os.chdir(git_root)
    return git_root


def has_uncommitted_changes() -> bool:
    """检查Git仓库中是否有未提交的更改

    返回:
        bool: 如果有未提交的更改返回True，否则返回False
    """
    # 静默添加所有更改
    subprocess.run(
        ["git", "add", "."], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL
    )

    # 检查工作目录更改
    working_changes = (
        subprocess.run(
            ["git", "diff", "--exit-code"],
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        ).returncode
        != 0
    )

    # 检查暂存区更改
    staged_changes = (
        subprocess.run(
            ["git", "diff", "--cached", "--exit-code"],
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        ).returncode
        != 0
    )

    # 静默重置更改
    subprocess.run(
        ["git", "reset"], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL
    )

    return working_changes or staged_changes


def get_commits_between(start_hash: str, end_hash: str) -> List[Tuple[str, str]]:
    """获取两个提交哈希值之间的提交列表

    参数：
        start_hash: 起始提交哈希值（不包含）
        end_hash: 结束提交哈希值（包含）

    返回：
        List[Tuple[str, str]]: (提交哈希值, 提交信息) 元组列表
    """
    try:
        # 使用git log和pretty格式获取哈希值和信息
        result = subprocess.run(
            ["git", "log", f"{start_hash}..{end_hash}", "--pretty=format:%H|%s"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=False,  # 禁用自动文本解码
        )
        if result.returncode != 0:
            error_msg = result.stderr.decode("utf-8", errors="replace")
            PrettyOutput.print(f"获取commit历史失败: {error_msg}", OutputType.ERROR)
            return []

        output = result.stdout.decode("utf-8", errors="replace")
        commits = []
        for line in output.splitlines():
            if "|" in line:
                commit_hash, message = line.split("|", 1)
                commits.append((commit_hash, message))
        return commits

    except Exception as e:
        PrettyOutput.print(f"获取commit历史异常: {str(e)}", OutputType.ERROR)
        return []


# 修改后的获取差异函数


def get_diff() -> str:
    """使用git获取工作区差异，包括修改和新增的文件内容

    返回:
        str: 差异内容或错误信息
    """
    try:
        # 检查是否为空仓库
        head_check = subprocess.run(
            ["git", "rev-parse", "--verify", "HEAD"],
            stderr=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        confirm_add_new_files()
        if head_check.returncode != 0:
            # 空仓库情况，直接获取工作区差异
            result = subprocess.run(
                ["git", "diff"], capture_output=True, text=False, check=True
            )
        else:
            # 暂存新增文件
            subprocess.run(["git", "add", "-N", "."], check=True)

            # 获取所有差异（包括新增文件）
            result = subprocess.run(
                ["git", "diff", "HEAD"], capture_output=True, text=False, check=True
            )

            # 重置暂存区
            subprocess.run(["git", "reset"], check=True)

        try:
            return result.stdout.decode("utf-8")
        except UnicodeDecodeError:
            return result.stdout.decode("utf-8", errors="replace")

    except subprocess.CalledProcessError as e:
        return f"获取差异失败: {str(e)}"
    except Exception as e:
        return f"发生意外错误: {str(e)}"


def revert_file(filepath: str) -> None:
    """增强版git恢复，处理新文件"""
    import subprocess

    try:
        # 检查文件是否在版本控制中
        result = subprocess.run(
            ["git", "ls-files", "--error-unmatch", filepath],
            stderr=subprocess.PIPE,
            text=False,  # 禁用自动文本解码
        )
        if result.returncode == 0:
            subprocess.run(["git", "checkout", "HEAD", "--", filepath], check=True)
        else:
            if os.path.exists(filepath):
                os.remove(filepath)
        subprocess.run(["git", "clean", "-f", "--", filepath], check=True)
    except subprocess.CalledProcessError as e:
        error_msg = e.stderr.decode("utf-8", errors="replace") if e.stderr else str(e)
        PrettyOutput.print(f"恢复文件失败: {error_msg}", OutputType.ERROR)


# 修改后的恢复函数


def revert_change() -> None:
    """恢复所有未提交的修改到HEAD状态"""
    import subprocess

    try:
        # 检查是否为空仓库
        head_check = subprocess.run(
            ["git", "rev-parse", "--verify", "HEAD"],
            stderr=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        if head_check.returncode == 0:
            subprocess.run(["git", "reset", "--hard", "HEAD"], check=True)
        subprocess.run(["git", "clean", "-fd"], check=True)
    except subprocess.CalledProcessError as e:
        PrettyOutput.print(f"恢复更改失败: {str(e)}", OutputType.ERROR)


def handle_commit_workflow() -> bool:
    """Handle the git commit workflow and return the commit details.

    Returns:
        bool: 提交是否成功
    """
    if is_confirm_before_apply_patch() and not user_confirm(
        "是否要提交代码？", default=True
    ):
        revert_change()
        return False

    import subprocess

    try:
        confirm_add_new_files()

        if not has_uncommitted_changes():
            return False

        # 获取当前分支的提交总数
        commit_result = subprocess.run(
            ["git", "rev-list", "--count", "HEAD"], capture_output=True, text=True
        )
        if commit_result.returncode != 0:
            return False

        commit_count = int(commit_result.stdout.strip())

        # 暂存所有修改
        subprocess.run(["git", "add", "."], check=True)

        # 提交变更
        subprocess.run(
            ["git", "commit", "-m", f"CheckPoint #{commit_count + 1}"], check=True
        )
        return True
    except subprocess.CalledProcessError as e:
        return False


def get_latest_commit_hash() -> str:
    """获取当前Git仓库的最新提交哈希值

    返回：
        str: 提交哈希值，如果不在Git仓库、空仓库或发生错误则返回空字符串
    """
    try:
        # 首先检查是否存在HEAD引用
        head_check = subprocess.run(
            ["git", "rev-parse", "--verify", "HEAD"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=False,
        )
        if head_check.returncode != 0:
            return ""  # 空仓库或无效HEAD

        # 获取HEAD的完整哈希值
        result = subprocess.run(
            ["git", "rev-parse", "HEAD"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=False,
        )
        return (
            result.stdout.decode("utf-8", errors="replace").strip()
            if result.returncode == 0
            else ""
        )
    except Exception:
        return ""


def get_modified_line_ranges() -> Dict[str, Tuple[int, int]]:
    """从Git差异中获取所有更改文件的修改行范围

    返回：
        字典，将文件路径映射到包含修改部分的（起始行, 结束行）范围元组。
        行号从1开始。
    """
    # 获取所有文件的Git差异
    diff_output = os.popen("git show").read()

    # 解析差异以获取修改的文件及其行范围
    result = {}
    current_file = None

    for line in diff_output.splitlines():
        # 匹配类似"+++ b/path/to/file"的行
        file_match = re.match(r"^\+\+\+ b/(.*)", line)
        if file_match:
            current_file = file_match.group(1)
            continue

        # 匹配类似"@@ -100,5 +100,7 @@"的行，其中+部分显示新行
        range_match = re.match(r"^@@ -\d+(?:,\d+)? \+(\d+)(?:,(\d+))? @@", line)
        if range_match and current_file:
            start_line = int(range_match.group(1))  # 保持从1开始
            line_count = int(range_match.group(2)) if range_match.group(2) else 1
            end_line = start_line + line_count - 1
            result[current_file] = (start_line, end_line)

    return result


def is_file_in_git_repo(filepath: str) -> bool:
    """检查文件是否在当前Git仓库中"""
    import subprocess

    try:
        # 获取Git仓库根目录
        repo_root = subprocess.run(
            ["git", "rev-parse", "--show-toplevel"], capture_output=True, text=True
        ).stdout.strip()

        # 检查文件路径是否在仓库根目录下
        return os.path.abspath(filepath).startswith(os.path.abspath(repo_root))
    except:
        return False


def check_and_update_git_repo(repo_path: str) -> bool:
    """检查并更新git仓库

    参数:
        repo_path: 仓库路径

    返回:
        bool: 是否执行了更新
    """
    # 检查上次检查日期
    last_check_file = os.path.join(get_data_dir(), "last_git_check")
    today_str = datetime.date.today().strftime("%Y-%m-%d")
    if os.path.exists(last_check_file):
        with open(last_check_file, "r") as f:
            last_check_date = f.read().strip()
        if last_check_date == today_str:
            return False

    curr_dir = os.path.abspath(os.getcwd())
    git_root = find_git_root_and_cd(repo_path)
    if git_root is None:
        return False

    try:
        # 检查是否有未提交的修改
        if has_uncommitted_changes():
            return False

        # 获取远程tag更新
        subprocess.run(["git", "fetch", "--tags"], cwd=git_root, check=True)
        # 获取最新本地tag
        local_tag_result = subprocess.run(
            ["git", "describe", "--tags", "--abbrev=0"],
            cwd=git_root,
            capture_output=True,
            text=True,
        )
        # 获取最新远程tag
        remote_tag_result = subprocess.run(
            ["git", "ls-remote", "--tags", "--refs", "origin"],
            cwd=git_root,
            capture_output=True,
            text=True,
        )
        if remote_tag_result.returncode == 0:
            # 提取最新的tag名称
            tags = [ref.split("/")[-1] for ref in remote_tag_result.stdout.splitlines()]
            tags = sorted(
                tags,
                key=lambda x: [
                    int(i) if i.isdigit() else i for i in re.split(r"([0-9]+)", x)
                ],
            )
            remote_tag = tags[-1] if tags else ""
            remote_tag_result.stdout = remote_tag

        if (
            local_tag_result.returncode == 0
            and remote_tag_result.returncode == 0
            and local_tag_result.stdout.strip() != remote_tag_result.stdout.strip()
        ):
            PrettyOutput.print(
                f"检测到新版本tag {remote_tag_result.stdout.strip()}，正在更新Jarvis...",
                OutputType.INFO,
            )
            subprocess.run(
                ["git", "checkout", remote_tag_result.stdout.strip()],
                cwd=git_root,
                check=True,
            )
            PrettyOutput.print(
                f"Jarvis已更新到tag {remote_tag_result.stdout.strip()}",
                OutputType.SUCCESS,
            )

            # 执行pip安装更新代码
            try:
                PrettyOutput.print("正在安装更新后的代码...", OutputType.INFO)

                # 检查是否在虚拟环境中
                in_venv = hasattr(sys, "real_prefix") or (
                    hasattr(sys, "base_prefix") and sys.base_prefix != sys.prefix
                )

                # 尝试普通安装
                install_cmd = [sys.executable, "-m", "pip", "install", "-e", "."]
                result = subprocess.run(
                    install_cmd, cwd=git_root, capture_output=True, text=True
                )

                if result.returncode == 0:
                    PrettyOutput.print("代码更新安装成功", OutputType.SUCCESS)
                    return True

                # 处理权限错误
                error_msg = result.stderr.strip()
                if not in_venv and (
                    "Permission denied" in error_msg or "not writeable" in error_msg
                ):
                    if user_confirm(
                        "检测到权限问题，是否尝试用户级安装(--user)？", True
                    ):
                        user_result = subprocess.run(
                            install_cmd + ["--user"],
                            cwd=git_root,
                            capture_output=True,
                            text=True,
                        )
                        if user_result.returncode == 0:
                            PrettyOutput.print("用户级代码安装成功", OutputType.SUCCESS)
                            return True
                        error_msg = user_result.stderr.strip()

                PrettyOutput.print(f"代码安装失败: {error_msg}", OutputType.ERROR)
                return False
            except Exception as e:
                PrettyOutput.print(
                    f"安装过程中发生意外错误: {str(e)}", OutputType.ERROR
                )
                return False
        # 更新检查日期文件
        with open(last_check_file, "w") as f:
            f.write(today_str)
        return False
    except Exception as e:
        PrettyOutput.print(f"Git仓库更新检查失败: {e}", OutputType.WARNING)
        return False
    finally:
        os.chdir(curr_dir)


def get_diff_file_list() -> List[str]:
    """获取HEAD到当前变更的文件列表，包括修改和新增的文件

    返回:
        List[str]: 修改和新增的文件路径列表
    """
    try:
        confirm_add_new_files()

        # 暂存新增文件
        subprocess.run(["git", "add", "-N", "."], check=True)

        # 获取所有差异文件（包括新增文件）
        result = subprocess.run(
            ["git", "diff", "--name-only", "HEAD"], capture_output=True, text=True
        )

        # 重置暂存区
        subprocess.run(["git", "reset"], check=True)

        if result.returncode != 0:
            PrettyOutput.print(
                f"获取差异文件列表失败: {result.stderr}", OutputType.ERROR
            )
            return []

        return [f for f in result.stdout.splitlines() if f]

    except subprocess.CalledProcessError as e:
        PrettyOutput.print(f"获取差异文件列表失败: {str(e)}", OutputType.ERROR)
        return []
    except Exception as e:
        PrettyOutput.print(f"获取差异文件列表异常: {str(e)}", OutputType.ERROR)
        return []


def get_recent_commits_with_files() -> List[Dict[str, Any]]:
    """获取最近5次提交的commit信息和文件清单

    返回:
        List[Dict[str, Any]]: 包含commit信息和文件清单的字典列表，格式为:
            [
                {
                    'hash': str,
                    'message': str,
                    'author': str,
                    'date': str,
                    'files': List[str]  # 修改的文件列表 (最多20个文件)
                },
                ...
            ]
            失败时返回空列表
    """
    try:
        # 获取当前git用户名
        current_author = subprocess.run(
            ["git", "config", "user.name"],
            capture_output=True,
            text=True,
        ).stdout.strip()

        # 获取当前用户最近5次提交的基本信息
        result = subprocess.run(
            [
                "git",
                "log",
                "-5",
                "--author=" + current_author,
                "--pretty=format:%H%n%s%n%an%n%ad",
            ],
            capture_output=True,
            text=True,
        )
        if result.returncode != 0:
            return []

        # 解析提交信息
        commits: List[Dict[str, Any]] = []
        lines = result.stdout.splitlines()
        for i in range(0, len(lines), 4):
            if i + 3 >= len(lines):
                break
            commit: Dict[str, Any] = {
                "hash": lines[i],
                "message": lines[i + 1],
                "author": lines[i + 2],
                "date": lines[i + 3],
                "files": [],
            }
            commits.append(commit)

        # 获取每个提交的文件修改清单
        for commit in commits:
            files_result = subprocess.run(
                ["git", "show", "--name-only", "--pretty=format:", commit["hash"]],
                capture_output=True,
                text=True,
            )
            if files_result.returncode == 0:
                file_lines = files_result.stdout.splitlines()
                unique_files: Set[str] = set(filter(None, file_lines))
                commit["files"] = list(unique_files)[:20]  # type: ignore[list-item] # 限制最多20个文件

        return commits

    except subprocess.CalledProcessError:
        return []


def _get_new_files() -> List[str]:
    """获取新增文件列表"""
    return subprocess.run(
        ["git", "ls-files", "--others", "--exclude-standard"],
        capture_output=True,
        text=True,
        check=True,
    ).stdout.splitlines()


def confirm_add_new_files() -> None:
    """确认新增文件、代码行数和二进制文件"""

    def _get_added_lines() -> int:
        """获取新增代码行数"""
        diff_stats = subprocess.run(
            ["git", "diff", "--numstat"],
            capture_output=True,
            text=True,
            check=True,
        ).stdout.splitlines()

        added_lines = 0
        for stat in diff_stats:
            parts = stat.split()
            if len(parts) >= 1:
                try:
                    added_lines += int(parts[0])
                except ValueError:
                    pass
        return added_lines

    def _get_binary_files(files: List[str]) -> List[str]:
        """从文件列表中识别二进制文件"""
        binary_files = []
        for file in files:
            try:
                with open(file, "rb") as f:
                    if b"\x00" in f.read(1024):
                        binary_files.append(file)
            except (IOError, PermissionError):
                continue
        return binary_files

    def _check_conditions(
        new_files: List[str], added_lines: int, binary_files: List[str]
    ) -> bool:
        """检查各种条件并打印提示信息"""
        need_confirm = False
        output_lines = []

        if len(new_files) > 20:
            output_lines.append(f"检测到{len(new_files)}个新增文件(选择N将重新检测)")
            output_lines.append("新增文件列表:")
            output_lines.extend(f"  - {file}" for file in new_files)
            need_confirm = True

        if added_lines > 500:
            output_lines.append(f"检测到{added_lines}行新增代码(选择N将重新检测)")
            need_confirm = True

        if binary_files:
            output_lines.append(
                f"检测到{len(binary_files)}个二进制文件(选择N将重新检测)"
            )
            output_lines.append("二进制文件列表:")
            output_lines.extend(f"  - {file}" for file in binary_files)
            need_confirm = True

        if output_lines:
            PrettyOutput.print(
                "\n".join(output_lines),
                OutputType.WARNING if need_confirm else OutputType.INFO,
            )

        return need_confirm

    while True:
        new_files = _get_new_files()
        added_lines = _get_added_lines()
        binary_files = _get_binary_files(new_files)

        if not _check_conditions(new_files, added_lines, binary_files):
            break

        if not user_confirm(
            "是否要添加这些变更（如果不需要请修改.gitignore文件以忽略不需要的文件）？",
            False,
        ):
            continue

        break



# -*- coding: utf-8 -*-
import fcntl
import os
import pty
import select
import signal
import time
from typing import Any, Dict


class VirtualTTYTool:
    name = "virtual_tty"
    description = (
        "控制虚拟终端执行各种操作，如启动终端、输入命令、获取输出等。"
        + "与execute_script不同，此工具会创建一个持久的虚拟终端会话，可以连续执行多个命令，并保持终端状态。"
        + "适用于需要交互式操作的场景，如运行需要用户输入的交互式程序（如：ssh连接、sftp传输、gdb/dlv调试等）。"
    )
    parameters = {
        "type": "object",
        "properties": {
            "action": {
                "type": "string",
                "description": "要执行的终端操作类型",
                "enum": [
                    "launch",
                    "send_keys",
                    "output",
                    "close",
                    "get_screen",
                    "list",
                ],
            },
            "keys": {
                "type": "string",
                "description": "要发送的按键序列（仅支持单行输入，当action为send_keys时有效）",
            },
            "add_enter": {
                "type": "boolean",
                "description": "是否在单行命令末尾自动添加回车符（仅当action为send_keys时有效，默认为true）",
            },
            "timeout": {
                "type": "number",
                "description": "等待输出的超时时间（秒，仅当action为send_keys或output时有效，默认为5.0）",
            },
            "tty_id": {
                "type": "string",
                "description": "虚拟终端的唯一标识符（默认为'default'）",
            },
        },
        "required": ["action"],
    }

    def execute(self, args: Dict[str, Any]) -> Dict[str, Any]:
        """执行终端操作

        参数:
            args: 包含操作参数的字典，包括agent属性

        返回:
            字典，包含以下内容：
                - success: 布尔值，表示操作状态
                - stdout: 成功消息或操作结果
                - stderr: 错误消息或空字符串
        """
        # 获取agent对象
        agent = args.get("agent")
        if agent is None:
            return {"success": False, "stdout": "", "stderr": "未提供agent对象"}

        # 获取TTY ID，默认为"default"
        tty_id = args.get("tty_id", "default")

        # 确保agent有tty_sessions字典
        if not hasattr(agent, "tty_sessions"):
            agent.tty_sessions = {}

        # 如果指定的tty_id不存在，为其创建一个新的tty_data
        if tty_id not in agent.tty_sessions:
            agent.tty_sessions[tty_id] = {
                "master_fd": None,
                "pid": None,
                "shell": "/bin/bash",
            }

        action = args.get("action", "").strip().lower()

        # 验证操作类型
        valid_actions = ["launch", "send_keys", "output", "close", "get_screen", "list"]
        if action not in valid_actions:
            return {
                "success": False,
                "stdout": "",
                "stderr": f"不支持的操作: {action}。有效操作: {', '.join(valid_actions)}",
            }

        try:
            if action == "launch":
                if args.get("keys", "") != "":
                    print(f"🚫 启动虚拟终端时，不能同时指定keys参数")
                    return {
                        "success": False,
                        "stdout": "",
                        "stderr": "启动虚拟终端时，不能同时指定keys参数",
                    }
                print(f"🚀 正在启动虚拟终端 [{tty_id}]...")
                result = self._launch_tty(agent, tty_id)
                if result["success"]:
                    print(f"✅ 启动虚拟终端 [{tty_id}] 成功")
                else:
                    print(f"❌ 启动虚拟终端 [{tty_id}] 失败")
                return result
            elif action == "send_keys":
                keys = args.get("keys", "").strip()
                add_enter = args.get("add_enter", True)
                timeout = args.get("timeout", 5.0)  # 默认5秒超时
                print(f"⌨️ 正在向终端 [{tty_id}] 发送按键序列: {keys}...")
                result = self._input_command(agent, tty_id, keys, timeout, add_enter)
                if result["success"]:
                    print(f"✅ 发送按键序列到终端 [{tty_id}] 成功")
                else:
                    print(f"❌ 发送按键序列到终端 [{tty_id}] 失败")
                return result
            elif action == "output":
                timeout = args.get("timeout", 5.0)  # 默认5秒超时
                print(f"📥 正在获取终端 [{tty_id}] 输出...")
                result = self._get_output(agent, tty_id, timeout)
                if result["success"]:
                    print(f"✅ 获取终端 [{tty_id}] 输出成功")
                else:
                    print(f"❌ 获取终端 [{tty_id}] 输出失败")
                return result
            elif action == "close":
                print(f"🔒 正在关闭虚拟终端 [{tty_id}]...")
                result = self._close_tty(agent, tty_id)
                if result["success"]:
                    print(f"✅ 关闭虚拟终端 [{tty_id}] 成功")
                else:
                    print(f"❌ 关闭虚拟终端 [{tty_id}] 失败")
                return result
            elif action == "get_screen":
                print(f"🖥️ 正在获取终端 [{tty_id}] 屏幕内容...")
                result = self._get_screen(agent, tty_id)
                if result["success"]:
                    print(f"✅ 获取终端 [{tty_id}] 屏幕内容成功")
                else:
                    print(f"❌ 获取终端 [{tty_id}] 屏幕内容失败")
                return result
            elif action == "list":
                print("📋 正在获取所有虚拟终端列表...")
                result = self._list_ttys(agent)
                if result["success"]:
                    print("✅ 获取虚拟终端列表成功")
                else:
                    print("❌ 获取虚拟终端列表失败")
                return result
            return {"success": False, "stdout": "", "stderr": "不支持的操作"}

        except Exception as e:
            return {
                "success": False,
                "stdout": "",
                "stderr": f"执行终端操作出错: {str(e)}",
            }

    def _launch_tty(self, agent: Any, tty_id: str) -> Dict[str, Any]:
        """启动虚拟终端"""
        try:
            # 如果该ID的终端已经启动，先关闭它
            if agent.tty_sessions[tty_id]["master_fd"] is not None:
                self._close_tty(agent, tty_id)

            # 创建伪终端
            pid, master_fd = pty.fork()

            if pid == 0:  # 子进程
                # 执行shell
                os.execvp(
                    agent.tty_sessions[tty_id]["shell"],
                    [agent.tty_sessions[tty_id]["shell"]],
                )
            else:  # 父进程
                # 设置非阻塞模式
                fcntl.fcntl(master_fd, fcntl.F_SETFL, os.O_NONBLOCK)

                # 保存终端状态
                agent.tty_sessions[tty_id]["master_fd"] = master_fd
                agent.tty_sessions[tty_id]["pid"] = pid

                # 读取初始输出
                output = ""
                start_time = time.time()
                while time.time() - start_time < 2.0:  # 最多等待2秒
                    try:
                        r, _, _ = select.select([master_fd], [], [], 0.1)
                        if r:
                            data = os.read(master_fd, 1024)
                            if data:
                                output += data.decode()
                    except BlockingIOError:
                        continue

                if output:
                    print(f"📤 终端 [{tty_id}]: {output}")

                return {"success": True, "stdout": output, "stderr": ""}

        except Exception as e:
            return {
                "success": False,
                "stdout": "",
                "stderr": f"启动虚拟终端 [{tty_id}] 失败: {str(e)}",
            }

    def _input_command(
        self,
        agent: Any,
        tty_id: str,
        command: str,
        timeout: float,
        add_enter: bool = True,
    ) -> Dict[str, Any]:
        """输入单行命令并等待输出

        参数:
            command: 要输入的单行命令
            add_enter: 是否在命令末尾添加回车符
        """
        if agent.tty_sessions[tty_id]["master_fd"] is None:
            return {
                "success": False,
                "stdout": "",
                "stderr": f"虚拟终端 [{tty_id}] 未启动",
            }

        # 严格检查并拒绝多行输入
        if "\n" in command:
            return {"success": False, "stdout": "", "stderr": "错误：禁止多行输入"}

        try:
            # 根据add_enter参数决定是否添加回车符
            if add_enter:
                command = command + "\n"

            # 发送按键序列
            os.write(agent.tty_sessions[tty_id]["master_fd"], command.encode())

            # 等待输出
            output = ""
            start_time = time.time()

            while time.time() - start_time < timeout:
                try:
                    # 使用select等待数据可读
                    r, _, _ = select.select(
                        [agent.tty_sessions[tty_id]["master_fd"]], [], [], 0.1
                    )
                    if r:
                        data = os.read(agent.tty_sessions[tty_id]["master_fd"], 1024)
                        if data:
                            output += data.decode()
                except BlockingIOError:
                    continue
            print(f"📤 终端 [{tty_id}]: {output}")
            return {"success": True, "stdout": output, "stderr": ""}

        except Exception as e:
            return {
                "success": False,
                "stdout": "",
                "stderr": f"在终端 [{tty_id}] 执行命令失败: {str(e)}",
            }

    def _get_output(
        self, agent: Any, tty_id: str, timeout: float = 5.0
    ) -> Dict[str, Any]:
        """获取终端输出"""
        if agent.tty_sessions[tty_id]["master_fd"] is None:
            return {
                "success": False,
                "stdout": "",
                "stderr": f"虚拟终端 [{tty_id}] 未启动",
            }

        try:
            output = ""
            start_time = time.time()

            while time.time() - start_time < timeout:
                # 使用select等待数据可读
                r, _, _ = select.select(
                    [agent.tty_sessions[tty_id]["master_fd"]], [], [], 0.1
                )
                if r:
                    while True:
                        try:
                            data = os.read(
                                agent.tty_sessions[tty_id]["master_fd"], 1024
                            )
                            if data:
                                output += data.decode()
                            else:
                                break
                        except BlockingIOError:
                            break
            print(f"📤 终端 [{tty_id}]: {output}")

            return {"success": True, "stdout": output, "stderr": ""}

        except Exception as e:
            return {
                "success": False,
                "stdout": "",
                "stderr": f"获取终端 [{tty_id}] 输出失败: {str(e)}",
            }

    def _close_tty(self, agent: Any, tty_id: str) -> Dict[str, Any]:
        """关闭虚拟终端"""
        if agent.tty_sessions[tty_id]["master_fd"] is None:
            return {
                "success": True,
                "stdout": f"没有正在运行的虚拟终端 [{tty_id}]",
                "stderr": "",
            }

        try:
            # 关闭主文件描述符
            os.close(agent.tty_sessions[tty_id]["master_fd"])

            # 终止子进程
            if agent.tty_sessions[tty_id]["pid"]:
                os.kill(agent.tty_sessions[tty_id]["pid"], signal.SIGTERM)

            # 重置终端数据
            agent.tty_sessions[tty_id] = {
                "master_fd": None,
                "pid": None,
                "shell": "/bin/bash",
            }

            return {
                "success": True,
                "stdout": f"虚拟终端 [{tty_id}] 已关闭",
                "stderr": "",
            }

        except Exception as e:
            return {
                "success": False,
                "stdout": "",
                "stderr": f"关闭虚拟终端 [{tty_id}] 失败: {str(e)}",
            }

    def _get_screen(self, agent: Any, tty_id: str) -> Dict[str, Any]:
        """获取当前终端屏幕内容"""
        if agent.tty_sessions[tty_id]["master_fd"] is None:
            return {
                "success": False,
                "stdout": "",
                "stderr": f"虚拟终端 [{tty_id}] 未启动",
            }

        try:
            # 发送控制序列获取屏幕内容
            os.write(
                agent.tty_sessions[tty_id]["master_fd"],
                b"\x1b[2J\x1b[H\x1b[999;999H\x1b[6n",
            )

            # 读取响应
            output = ""
            start_time = time.time()
            while time.time() - start_time < 2.0:  # 最多等待2秒
                try:
                    r, _, _ = select.select(
                        [agent.tty_sessions[tty_id]["master_fd"]], [], [], 0.1
                    )
                    if r:
                        data = os.read(agent.tty_sessions[tty_id]["master_fd"], 1024)
                        if data:
                            output += data.decode()
                except BlockingIOError:
                    continue

            # 清理控制字符
            output = (
                output.replace("\x1b[2J", "")
                .replace("\x1b[H", "")
                .replace("\x1b[999;999H", "")
                .replace("\x1b[6n", "")
            )

            return {"success": True, "stdout": output.strip(), "stderr": ""}

        except Exception as e:
            return {
                "success": False,
                "stdout": "",
                "stderr": f"获取终端 [{tty_id}] 屏幕内容失败: {str(e)}",
            }

    def _list_ttys(self, agent: Any) -> Dict[str, Any]:
        """列出所有虚拟终端"""
        try:
            active_ttys = []

            for tty_id, tty_data in agent.tty_sessions.items():
                status = "活动" if tty_data["master_fd"] is not None else "关闭"
                active_ttys.append(
                    {
                        "id": tty_id,
                        "status": status,
                        "pid": tty_data["pid"] if tty_data["pid"] else None,
                        "shell": tty_data["shell"],
                    }
                )

            # 格式化输出
            output = "虚拟终端列表:\n"
            for tty in active_ttys:
                output += f"ID: {tty['id']}, 状态: {tty['status']}, PID: {tty['pid']}, Shell: {tty['shell']}\n"

            return {
                "success": True,
                "stdout": output,
                "stderr": "",
                "tty_list": active_ttys,  # 返回原始数据，方便程序处理
            }

        except Exception as e:
            return {
                "success": False,
                "stdout": "",
                "stderr": f"获取虚拟终端列表失败: {str(e)}",
            }



# -*- coding: utf-8 -*-
"""
DevOps (Makefile, CI/CD) specific code review checklist.
"""

CHECKLIST = """
## DevOps (Makefile, CI/CD) 代码审查检查清单

### Makefile 结构与风格
- [ ] 目标是否清晰且有明确用途
- [ ] 是否使用了合适的依赖关系
- [ ] 是否有默认目标和帮助信息
- [ ] 变量命名是否一致且有意义
- [ ] 是否使用了适当的变量替换
- [ ] 是否避免了重复代码
- [ ] 目标规则是否组织良好并有注释
- [ ] 是否使用了模式规则和自动变量

### Makefile 最佳实践
- [ ] 是否使用了.PHONY伪目标
- [ ] 是否声明了必要的环境变量
- [ ] 是否使用了适当的错误处理
- [ ] 是否优化了构建性能
- [ ] 是否支持并行构建 (-j 选项)
- [ ] 是否考虑了跨平台兼容性
- [ ] 是否适当使用了条件逻辑
- [ ] 是否包含了清理目标

### CI/CD 配置审查
- [ ] 工作流程是否清晰定义
- [ ] 是否使用了合适的触发条件
- [ ] 是否正确定义了环境和依赖
- [ ] 是否有适当的阶段和步骤划分
- [ ] 是否实现了合理的并行执行
- [ ] 是否有适当的缓存策略
- [ ] 是否处理了错误和恢复机制
- [ ] 是否包含了适当的通知配置

### 环境管理
- [ ] 是否明确定义了所需的环境变量
- [ ] 敏感信息是否安全管理
- [ ] 是否有环境隔离 (开发/测试/生产)
- [ ] 是否有版本锁定机制
- [ ] 是否考虑了可重现性
- [ ] 是否有适当的环境验证步骤
- [ ] 是否考虑了可扩展性需求
- [ ] 依赖管理是否合理

### 自动化构建
- [ ] 构建步骤是否完整且按顺序
- [ ] 构建过程是否可重现
- [ ] 是否包含了必要的验证步骤
- [ ] 资源使用是否优化
- [ ] 是否有适当的错误报告
- [ ] 是否包含了版本控制信息
- [ ] 构建成品是否明确定义
- [ ] 是否有适当的构建日志

### 测试自动化
- [ ] 是否包含了单元测试
- [ ] 是否包含了集成测试
- [ ] 是否设置了适当的测试环境
- [ ] 测试结果是否易于解读
- [ ] 是否避免了测试之间的依赖
- [ ] 测试覆盖率是否合理
- [ ] 是否有失败测试的恢复机制
- [ ] 测试执行是否优化

### 部署流程
- [ ] 部署步骤是否明确且按顺序
- [ ] 是否有回滚机制
- [ ] 是否使用了蓝绿部署或金丝雀发布
- [ ] 是否进行了部署前验证
- [ ] 是否有部署后验证
- [ ] 是否有适当的权限控制
- [ ] 是否支持不同环境的部署
- [ ] 是否有部署拆分或分阶段发布机制

### 安全考虑
- [ ] 构建和部署脚本是否避免了硬编码凭证
- [ ] 是否有适当的访问控制
- [ ] 是否进行了安全扫描
- [ ] 依赖项是否经过安全审查
- [ ] 是否避免了注入漏洞
- [ ] 临时凭证管理是否安全
- [ ] 是否有安全合规检查
- [ ] 是否记录了安全相关日志

### 监控和可观测性
- [ ] 是否包含了健康检查
- [ ] 是否实现了适当的日志记录
- [ ] 是否有性能监控
- [ ] 是否有错误追踪和报警
- [ ] 是否考虑了可追踪性
- [ ] 是否有适当的指标收集
- [ ] 是否考虑了日志聚合
- [ ] 是否有状态报告机制

### 文档和维护
- [ ] 是否有清晰的使用文档
- [ ] 命令和选项是否有描述
- [ ] 是否记录了关键决策和注意事项
- [ ] 脚本是否易于维护和扩展
- [ ] 是否有明确的维护责任
- [ ] 是否有版本控制和变更历史
- [ ] 是否有明确的故障排除指南
- [ ] 依赖关系是否明确记录
"""



# -*- coding: utf-8 -*-
import hashlib
import json
import os
from typing import Any, Dict

from jarvis.jarvis_utils.config import get_data_dir
from jarvis.jarvis_utils.output import OutputType, PrettyOutput


class MethodologyTool:
    """经验管理工具"""

    name = "methodology"
    description = "管理问题解决方法论，支持添加、更新和删除操作"
    parameters = {
        "type": "object",
        "properties": {
            "operation": {
                "type": "string",
                "description": "操作类型（delete/update/add）",
                "enum": ["delete", "update", "add"],
            },
            "problem_type": {
                "type": "string",
                "description": "问题类型，例如：部署开源项目、生成提交信息",
            },
            "content": {
                "type": "string",
                "description": "方法论内容（更新和添加时必填）",
                "optional": True,
            },
        },
        "required": ["operation", "problem_type"],
    }

    def __init__(self):
        """初始化经验管理工具"""
        self.methodology_dir = os.path.join(get_data_dir(), "methodologies")
        self._ensure_dir_exists()

    def _ensure_dir_exists(self):
        """确保方法论目录存在"""
        if not os.path.exists(self.methodology_dir):
            try:
                os.makedirs(self.methodology_dir, exist_ok=True)
            except Exception as e:
                PrettyOutput.print(f"创建方法论目录失败：{str(e)}", OutputType.ERROR)

    def _get_methodology_file_path(self, problem_type: str) -> str:
        """
        根据问题类型获取对应的方法论文件路径

        参数:
            problem_type: 问题类型

        返回:
            str: 方法论文件路径
        """
        # 使用MD5哈希作为文件名，避免文件名中的特殊字符
        safe_filename = hashlib.md5(problem_type.encode("utf-8")).hexdigest()
        return os.path.join(self.methodology_dir, f"{safe_filename}.json")

    def execute(self, args: Dict[str, Any]) -> Dict[str, Any]:
        """执行管理方法论的操作

        Args:
            args: 包含操作参数的字典
                - operation: 操作类型 (delete/update/add)
                - problem_type: 问题类型
                - content: 方法论内容 (更新和添加时必填)

        Returns:
            Dict[str, Any]: 包含执行结果的字典
        """
        operation = args.get("operation", "").strip()
        problem_type = args.get("problem_type", "").strip()
        content = args.get("content", "").strip()

        if not operation or not problem_type:
            return {
                "success": False,
                "stdout": "",
                "stderr": "缺少必要参数: operation和problem_type",
            }

        try:
            if operation == "delete":
                # 获取方法论文件路径
                file_path = self._get_methodology_file_path(problem_type)

                # 检查文件是否存在
                if os.path.exists(file_path):
                    os.remove(file_path)
                    return {
                        "success": True,
                        "stdout": f"已删除问题类型'{problem_type}'对应的方法论",
                        "stderr": "",
                    }
                else:
                    return {
                        "success": False,
                        "stdout": "",
                        "stderr": f"未找到问题类型'{problem_type}'对应的方法论",
                    }

            elif operation in ["update", "add"]:
                if not content:
                    return {
                        "success": False,
                        "stdout": "",
                        "stderr": "需要提供方法论内容",
                    }

                # 确保目录存在
                self._ensure_dir_exists()

                # 获取方法论文件路径
                file_path = self._get_methodology_file_path(problem_type)

                # 保存方法论到单独的文件
                with open(file_path, "w", encoding="utf-8", errors="ignore") as f:
                    json.dump(
                        {"problem_type": problem_type, "content": content},
                        f,
                        ensure_ascii=False,
                        indent=2,
                    )

                PrettyOutput.print(f"方法论已保存到 {file_path}", OutputType.INFO)

                action = "更新" if os.path.exists(file_path) else "添加"
                return {
                    "success": True,
                    "stdout": f"{action}了问题类型'{problem_type}'对应的方法论",
                    "stderr": "",
                }

            else:
                return {
                    "success": False,
                    "stdout": "",
                    "stderr": f"不支持的操作类型: {operation}",
                }

        except Exception as e:
            return {"success": False, "stdout": "", "stderr": f"执行失败: {str(e)}"}



# -*- coding: utf-8 -*-
import os
from typing import Any, Dict

from jarvis.jarvis_utils.output import OutputType, PrettyOutput


class ReadCodeTool:
    name = "read_code"
    description = "代码阅读与分析工具，用于读取源代码文件并添加行号，针对代码文件优化，提供更好的格式化输出和行号显示，适用于代码分析、审查和理解代码实现的场景"
    # 工具标签
    parameters = {
        "type": "object",
        "properties": {
            "files": {
                "type": "array",
                "items": {
                    "type": "object",
                    "properties": {
                        "path": {"type": "string"},
                        "start_line": {"type": "number", "default": 1},
                        "end_line": {"type": "number", "default": -1},
                    },
                    "required": ["path"],
                },
                "description": "要读取的文件列表",
            }
        },
        "required": ["files"],
    }

    def _handle_single_file(
        self, filepath: str, start_line: int = 1, end_line: int = -1, agent: Any = None
    ) -> Dict[str, Any]:
        """处理单个文件的读取操作

        Args:
            filepath (str): 文件路径
            start_line (int): 起始行号，默认为1
            end_line (int): 结束行号，默认为-1表示文件末尾

        Returns:
            Dict[str, Any]: 包含成功状态、输出内容和错误信息的字典
        """
        try:
            abs_path = os.path.abspath(filepath)
            print(f"📖 正在读取文件: {abs_path}...")
            # 文件存在性检查
            if not os.path.exists(abs_path):
                return {
                    "success": False,
                    "stdout": "",
                    "stderr": f"文件不存在: {abs_path}",
                }

            # 文件大小限制检查（10MB）
            if os.path.getsize(abs_path) > 10 * 1024 * 1024:
                return {
                    "success": False,
                    "stdout": "",
                    "stderr": "文件过大 (>10MB)",
                }

            # 读取文件内容
            with open(abs_path, "r", encoding="utf-8", errors="ignore") as f:
                lines = f.readlines()

            total_lines = len(lines)

            # 处理空文件情况
            if total_lines == 0:
                print(f"✅ 文件读取完成: {abs_path}")
                return {
                    "success": True,
                    "stdout": f"\n🔍 文件: {abs_path}\n📄 文件为空 (0行)\n",
                    "stderr": "",
                }

            # 处理特殊值-1表示文件末尾
            if end_line == -1:
                end_line = total_lines
            else:
                end_line = (
                    max(1, min(end_line, total_lines))
                    if end_line >= 0
                    else total_lines + end_line + 1
                )

            start_line = (
                max(1, min(start_line, total_lines))
                if start_line >= 0
                else total_lines + start_line + 1
            )

            if start_line > end_line:
                print(
                    f"❌ 无效的行范围 [{start_line}-{end_line}] (总行数: {total_lines})"
                )
                return {
                    "success": False,
                    "stdout": "",
                    "stderr": f"无效的行范围 [{start_line}-{end_line}] (总行数: {total_lines})",
                }

            # 添加行号并构建输出内容
            selected_lines = lines[start_line - 1 : end_line]
            numbered_content = "".join(
                [
                    f"{i:4d}:{line}"
                    for i, line in enumerate(selected_lines, start=start_line)
                ]
            )

            # 构建输出格式
            output = (
                f"\n🔍 文件: {abs_path}\n"
                f"📄 原始行号: {start_line}-{end_line} (共{total_lines}行) \n\n"
                f"{numbered_content}\n\n"
            )
            print(f"✅ 文件读取完成: {abs_path}")

            if agent:
                files = agent.get_user_data("files")
                if files:
                    files.append(abs_path)
                else:
                    files = [abs_path]
                agent.set_user_data("files", files)

            return {"success": True, "stdout": output, "stderr": ""}

        except Exception as e:
            PrettyOutput.print(str(e), OutputType.ERROR)
            return {"success": False, "stdout": "", "stderr": f"文件读取失败: {str(e)}"}

    def execute(self, args: Dict) -> Dict[str, Any]:
        """执行代码读取操作

        Args:
            args (Dict): 包含文件列表的参数字典

        Returns:
            Dict[str, Any]: 包含成功状态、输出内容和错误信息的字典
        """
        try:
            agent = args.get("agent", None)
            if "files" not in args or not isinstance(args["files"], list):
                return {
                    "success": False,
                    "stdout": "",
                    "stderr": "参数中必须包含文件列表",
                }

            all_outputs = []
            overall_success = True

            for file_info in args["files"]:
                if not isinstance(file_info, dict) or "path" not in file_info:
                    continue

                result = self._handle_single_file(
                    file_info["path"].strip(),
                    file_info.get("start_line", 1),
                    file_info.get("end_line", -1),
                    agent,
                )

                if result["success"]:
                    all_outputs.append(result["stdout"])
                else:
                    all_outputs.append(f"❌ {file_info['path']}: {result['stderr']}")
                    overall_success = False

            return {
                "success": overall_success,
                "stdout": "\n".join(all_outputs),
                "stderr": "",
            }

        except Exception as e:
            PrettyOutput.print(str(e), OutputType.ERROR)
            return {"success": False, "stdout": "", "stderr": f"代码读取失败: {str(e)}"}



# -*- coding: utf-8 -*-
# 导入所需的类型注解模块
from typing import Any, Dict

# 导入多行输入工具和输出工具
from jarvis.jarvis_utils.output import OutputType, PrettyOutput


# 定义AskUserTool类，用于向用户提问
class AskUserTool:
    name = "ask_user"
    description = """当完成任务所需的信息缺失或关键决策信息不足时，向用户提问。用户可以输入多行文本，以空行结束。使用场景：1. 需要用户提供更多信息以完成任务；2. 需要用户做出关键决策；3. 需要用户确认重要操作；4. 需要用户提供额外信息"""
    # 定义参数结构，指定必须包含的问题字段
    parameters = {
        "type": "object",
        "properties": {
            "question": {"type": "string", "description": "要向用户提出的问题"}
        },
        "required": ["question"],
    }

    def execute(self, args: Dict[str, Any]) -> Dict[str, Any]:
        """执行向用户提问的操作

        Args:
            args: 一个包含问题的字典

        Returns:
            Dict: 一个包含用户响应的字典
        """
        try:
            # 从参数中获取问题
            question = args["question"]

            # 获取agent对象并重置工具调用计数
            agent = args["agent"]
            agent.set_run_input_handlers_next_turn(True)

            # 显示问题给用户
            PrettyOutput.print(f"问题: {question}", OutputType.SYSTEM)

            # 获取用户输入
            user_response = agent.multiline_inputer("请输入您的答案 (输入空行结束)")

            # 返回成功响应，包含用户输入的内容
            return {"success": True, "stdout": user_response, "stderr": ""}

        except Exception as e:
            # 如果发生异常，返回失败响应，包含错误信息
            return {
                "success": False,
                "stdout": "",
                "stderr": f"Failed to ask user: {str(e)}",
            }



# -*- coding: utf-8 -*-
from typing import Optional

import typer
import yaml

from jarvis.jarvis_multi_agent import MultiAgent
from jarvis.jarvis_utils.input import get_multiline_input
from jarvis.jarvis_utils.utils import init_env

app = typer.Typer(help="多智能体系统启动器")


@app.command()
def cli(
    config: str = typer.Option(..., "--config", "-c", help="YAML配置文件路径"),
    user_input: Optional[str] = typer.Option(None, "--input", "-i", help="用户输入（可选）"),
):
    """从YAML配置文件初始化并运行多智能体系统"""
    init_env("欢迎使用 Jarvis-MultiAgent，您的多智能体系统已准备就绪！")

    try:
        with open(config, "r", errors="ignore") as f:
            config_data = yaml.safe_load(f)

        # 获取agents配置
        agents_config = config_data.get("agents", [])

        main_agent_name = config_data.get("main_agent", "")
        if not main_agent_name:
            raise ValueError("必须指定main_agent作为主智能体")

        # 创建并运行多智能体系统
        multi_agent = MultiAgent(agents_config, main_agent_name)
        final_input = (
            user_input
            if user_input is not None
            else get_multiline_input("请输入内容（输入空行结束）：")
        )
        if not final_input:
            raise typer.Exit(code=0)
        multi_agent.run(final_input)

    except typer.Exit:
        raise
    except (ValueError, RuntimeError, yaml.YAMLError) as e:
        PrettyOutput.print(f"错误: {str(e)}", OutputType.ERROR)
        raise typer.Exit(code=1)


def main() -> None:
    """Application entry point."""
    app()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
import unicodedata


class FileProcessor:
    """Base class for file processor"""

    @staticmethod
    def can_handle(file_path: str) -> bool:
        """Determine if the file can be processed"""
        raise NotImplementedError

    @staticmethod
    def extract_text(file_path: str) -> str:
        """Extract file text content"""
        raise NotImplementedError


class TextFileProcessor(FileProcessor):
    """Text file processor"""

    ENCODINGS = ["utf-8", "gbk", "gb2312", "latin1"]
    SAMPLE_SIZE = 8192  # Read the first 8KB to detect encoding

    @staticmethod
    def can_handle(file_path: str) -> bool:
        """Determine if the file is a text file by trying to decode it"""
        try:
            # Read the first part of the file to detect encoding
            with open(file_path, "rb") as f:
                sample = f.read(TextFileProcessor.SAMPLE_SIZE)

            # Check if it contains null bytes (usually represents a binary file)
            if b"\x00" in sample:
                return False

            # Check if it contains too many non-printable characters (usually represents a binary file)
            non_printable = sum(
                1 for byte in sample if byte < 32 and byte not in (9, 10, 13)
            )  # tab, newline, carriage return
            if (
                non_printable / len(sample) > 0.3
            ):  # If non-printable characters exceed 30%, it is considered a binary file
                return False

            # Try to decode with different encodings
            for encoding in TextFileProcessor.ENCODINGS:
                try:
                    sample.decode(encoding)
                    return True
                except UnicodeDecodeError:
                    continue

            return False

        except Exception:
            return False

    @staticmethod
    def extract_text(file_path: str) -> str:
        """Extract text content, using the detected correct encoding"""
        detected_encoding = None
        try:
            # First try to detect encoding
            with open(file_path, "rb") as f:
                raw_data = f.read()

            # Try different encodings
            for encoding in TextFileProcessor.ENCODINGS:
                try:
                    raw_data.decode(encoding)
                    detected_encoding = encoding
                    break
                except UnicodeDecodeError:
                    continue

            if not detected_encoding:
                raise UnicodeDecodeError(f"Failed to decode file with supported encodings: {file_path}")  # type: ignore

            # Use the detected encoding to read the file
            with open(file_path, "r", encoding=detected_encoding, errors="ignore") as f:
                content = f.read()

            # Normalize Unicode characters
            content = unicodedata.normalize("NFKC", content)

            return content

        except Exception as e:
            raise Exception(f"Failed to read file: {str(e)}")



# -*- coding: utf-8 -*-
"""
Shell/Bash language-specific code review checklist.
"""

CHECKLIST = """
## Shell/Bash 代码审查检查清单

### 脚本结构和可读性
- [ ] 是否有明确的 shebang 行 (#!/bin/bash 或 #!/usr/bin/env bash)
- [ ] 是否有脚本描述和使用说明
- [ ] 代码是否有一致的缩进和格式
- [ ] 函数和代码块是否有注释说明
- [ ] 命名是否清晰表达意图
- [ ] 是否使用 shellcheck 等工具检查代码

### 错误处理和调试
- [ ] 是否设置了 `set -e` 或 `set -o errexit`
- [ ] 是否适当处理管道失败 (`set -o pipefail`)
- [ ] 是否处理未声明变量 (`set -u` 或 `set -o nounset`)
- [ ] 是否有适当的错误消息和日志
- [ ] 退出代码 (exit codes) 是否一致且有意义
- [ ] 是否考虑了错误路径和异常情况

### 命令和管道
- [ ] 复杂管道是否可读和可维护
- [ ] 是否避免了不必要的命令和处理步骤
- [ ] 是否考虑了命令失败的情况
- [ ] 是否避免了不必要的子 shell
- [ ] 是否正确使用命令替换 ($(...) 而非 `...`)

### 变量和参数
- [ ] 变量引用是否使用双引号 ("$var" 而非 $var)
- [ ] 是否避免了全局变量污染
- [ ] 是否使用有意义的变量名
- [ ] 是否正确处理特殊字符和空格
- [ ] 是否验证命令行参数
- [ ] 是否设置了默认值 (${var:-default})
- [ ] 常量是否使用大写命名

### 函数
- [ ] 是否适当使用函数模块化代码
- [ ] 函数是否有明确的输入和输出
- [ ] 函数是否有文档说明
- [ ] 函数是否是幂等的 (可重复运行)
- [ ] 是否避免了副作用
- [ ] 返回状态是否明确

### 安全性
- [ ] 是否安全处理用户输入
- [ ] 是否避免了命令注入风险
- [ ] 敏感信息是否安全管理 (不使用明文密码)
- [ ] 临时文件是否安全创建和清理
- [ ] 是否正确设置了文件权限
- [ ] 是否避免了 eval 和其他危险结构

### 可移植性
- [ ] 是否考虑了不同 shell 的兼容性 (bash, sh, zsh)
- [ ] 是否避免了系统特定的命令和功能
- [ ] 是否考虑了路径的可移植性
- [ ] 是否考虑了不同环境的差异 (Linux, macOS)
- [ ] 外部依赖是否明确记录

### 性能
- [ ] 是否避免了不必要的外部命令调用
- [ ] 循环是否高效
- [ ] 是否优化了文件 I/O 操作
- [ ] 是否避免了不必要的重复计算
- [ ] 是否合理使用并行处理

### 可测试性
- [ ] 代码是否易于测试
- [ ] 是否有自动化测试
- [ ] 测试是否涵盖主要功能和错误路径
- [ ] 是否可以使用 mock 替换外部依赖
"""



import os
from typing import Any, Dict, Generator, List, Tuple

from jarvis.jarvis_platform.base import BasePlatform
import json

from jarvis.jarvis_utils import http
from jarvis.jarvis_utils.output import OutputType, PrettyOutput
from jarvis.jarvis_utils.utils import while_success


class AI8Model(BasePlatform):
    """AI8 model implementation"""

    BASE_URL = "https://ai8.rcouyi.com"

    def get_model_list(self) -> List[Tuple[str, str]]:
        """获取模型列表"""
        self.get_available_models()
        return [(name, info["desc"]) for name, info in self.models.items()]

    def __init__(self):
        """Initialize model"""
        super().__init__()
        self.system_prompt = ""
        self.conversation = {}
        self.models = {}  # 存储模型信息

        self.token = os.getenv("AI8_API_KEY")
        if not self.token:
            PrettyOutput.print("未设置 AI8_API_KEY", OutputType.WARNING)

        self.headers = {
            "Authorization": self.token,
            "sec-ch-ua-platform": '"Windows"',
            "sec-ch-ua": '"Not)A;Brand";v="8", "Chromium";v="138", "Google Chrome";v="138"',
            "sec-ch-ua-mobile": "?0",
            "Content-Type": "application/json",
            "Accept": "application/json, text/plain, */*",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/138.0.0.0 Safari/537.36",
            "X-APP-VERSION": "2.4.2",
            "Origin": self.BASE_URL,
            "Referer": f"{self.BASE_URL}/chat?_userMenuKey=chat",
            "Sec-Fetch-Site": "same-origin",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Dest": "empty",
            "Accept-Language": "zh-CN,zh;q=0.9",
            "Accept-Encoding": "gzip, deflate, br, zstd",
            "Connection": "keep-alive",
        }

        self.model_name = os.getenv("JARVIS_MODEL") or "deepseek-chat"
        if self.model_name not in self.get_available_models():
            PrettyOutput.print(
                f"警告: 选择的模型 {self.model_name} 不在可用列表中", OutputType.WARNING
            )

    def set_model_name(self, model_name: str):
        """Set model name"""

        self.model_name = model_name

    def create_conversation(self) -> bool:
        """Create a new conversation"""
        try:
            # 1. 创建会话
            response = while_success(
                lambda: http.post(
                    f"{self.BASE_URL}/api/chat/session",
                    headers=self.headers,
                    json={
                        "mcp": [],
                        "model": self.model_name,
                        "plugins": [],
                        "rags": [],
                    },
                ),
                sleep_time=5,
            )

            data = response.json()
            if data["code"] != 0:
                PrettyOutput.print(
                    f"创建会话失败: {data.get('msg', '未知错误')}", OutputType.WARNING
                )
                return False

            self.conversation = data["data"]

            # 2. 更新会话设置
            session_data = {
                **self.conversation,
                "model": self.model_name,
                "contextCount": 65536,
                "prompt": self.system_prompt,
                "plugins": [],
                "localPlugins": None,
                "useAppId": 0,
                "temperature": 0,
            }

            response = while_success(
                lambda: http.put(
                    f"{self.BASE_URL}/api/chat/session/{self.conversation['id']}",  # type: ignore
                    headers=self.headers,
                    json=session_data,
                ),
                sleep_time=5,
            )

            data = response.json()
            if data["code"] == 0:
                self.conversation = data["data"]
                return True
            else:
                PrettyOutput.print(
                    f"更新会话设置失败: {data.get('msg', '未知错误')}",
                    OutputType.WARNING,
                )
                return False

        except Exception as e:
            PrettyOutput.print(f"创建会话失败: {str(e)}", OutputType.ERROR)
            return False

    def set_system_prompt(self, message: str):
        """Set system message"""
        self.system_prompt = message

    def chat(self, message: str) -> Generator[str, None, None]:
        """Execute conversation"""
        try:
            # 确保有会话ID
            if not self.conversation:
                if not self.create_conversation():
                    raise Exception("Failed to create conversation")

            payload: Dict[str, Any] = {
                "text": message,
                "sessionId": self.conversation["id"] if self.conversation else None,
                "files": [],
            }

            # 为流式请求构造专用的请求头，避免 'Accept' 和 'accept' 键冲突
            stream_headers = self.headers.copy()
            stream_headers["Accept"] = "text/event-stream"  # 添加流式专用的accept头

            # 使用stream_post进行流式请求
            response_stream = while_success(
                lambda: http.stream_post(
                    f"{self.BASE_URL}/api/chat/completions",
                    headers=stream_headers,
                    json=payload,
                ),
                sleep_time=5,
            )

            # 处理流式响应
            for line in response_stream:
                if line and line.startswith("data: "):
                    try:
                        data = json.loads(line[6:])
                        if data.get("type") == "string":
                            chunk_data = data.get("data", "")
                            if chunk_data:
                                yield chunk_data
                    except json.JSONDecodeError:
                        continue

            return None

        except Exception as e:
            PrettyOutput.print(f"对话异常: {str(e)}", OutputType.ERROR)
            raise e

    def name(self) -> str:
        """Return model name"""
        return self.model_name

    @classmethod
    def platform_name(cls) -> str:
        """Return platform name"""
        return "ai8"

    def delete_chat(self) -> bool:
        """Delete current chat session"""
        try:
            if not self.conversation:
                return True

            response = while_success(
                lambda: http.delete(
                    f"{self.BASE_URL}/api/chat/session/{self.conversation['id']}",  # type: ignore
                    headers=self.headers,
                ),
                sleep_time=5,
            )

            data = response.json()
            if data["code"] == 0:
                self.conversation = {}
                return True
            else:
                error_msg = f"删除会话失败: {data.get('msg', '未知错误')}"
                PrettyOutput.print(error_msg, OutputType.WARNING)
                return False

        except Exception as e:
            PrettyOutput.print(f"删除会话失败: {str(e)}", OutputType.ERROR)
            return False

    def save(self, file_path: str) -> bool:
        """Save chat session to a file."""
        if not self.conversation:
            PrettyOutput.print("没有活动的会话可供保存", OutputType.WARNING)
            return False

        state = {
            "conversation": self.conversation,
            "model_name": self.model_name,
            "system_prompt": self.system_prompt,
        }

        try:
            with open(file_path, "w", encoding="utf-8") as f:
                json.dump(state, f, ensure_ascii=False, indent=4)
            self._saved = True
            PrettyOutput.print(f"会话已成功保存到 {file_path}", OutputType.SUCCESS)
            return True
        except Exception as e:
            PrettyOutput.print(f"保存会话失败: {str(e)}", OutputType.ERROR)
            return False

    def restore(self, file_path: str) -> bool:
        """Restore chat session from a file."""
        try:
            with open(file_path, "r", encoding="utf-8") as f:
                state = json.load(f)

            self.conversation = state["conversation"]
            self.model_name = state["model_name"]
            self.system_prompt = state.get("system_prompt", "")

            # A restored session should not be deleted on exit, as it's persistent.
            self._saved = True

            PrettyOutput.print(f"从 {file_path} 成功恢复会话", OutputType.SUCCESS)
            return True
        except FileNotFoundError:
            PrettyOutput.print(f"会话文件未找到: {file_path}", OutputType.ERROR)
            return False
        except KeyError as e:
            PrettyOutput.print(f"恢复失败: 会话文件格式不正确，缺少键 {e}", OutputType.ERROR)
            return False
        except Exception as e:
            PrettyOutput.print(f"恢复会话失败: {str(e)}", OutputType.ERROR)
            return False

    def get_available_models(self) -> List[str]:
        """Get available model list

        Returns:
            List[str]: Available model name list
        """
        try:
            if self.models:
                return list(self.models.keys())

            response = while_success(
                lambda: http.get(
                    f"{self.BASE_URL}/api/chat/tmpl", headers=self.headers
                ),
                sleep_time=5,
            )

            data = response.json()
            if data["code"] != 0:
                PrettyOutput.print(
                    f"获取模型列表失败: {data.get('msg', '未知错误')}",
                    OutputType.WARNING,
                )
                return []

            # 保存模型信息
            self.models = {model["value"]: model for model in data["data"]["models"]}

            for model in self.models.values():
                # 添加标签
                model_str = f"{model['label']}"

                # 添加特性标记
                features = []
                if model["attr"].get("multimodal"):
                    features.append("Multimodal")
                if model["attr"].get("plugin"):
                    features.append("Plugin support")
                if model["attr"].get("onlyImg"):
                    features.append("Image support")
                if model["attr"].get("tag"):
                    features.append(model["attr"]["tag"])
                if model["attr"].get("integral"):
                    features.append(model["attr"]["integral"])
                # 添加备注
                if model["attr"].get("note"):
                    model_str += f" - {model['attr']['note']}"
                if features:
                    model_str += f" [{'|'.join(features)}]"

                model["desc"] = model_str

            return list(self.models.keys())

        except Exception as e:
            PrettyOutput.print(f"获取模型列表失败: {str(e)}", OutputType.ERROR)
            return []

    def support_upload_files(self) -> bool:
        return False

    def support_web(self) -> bool:
        return False

    def upload_files(self, file_list: List[str]) -> bool:
        return False



# -*- coding: utf-8 -*-
"""
C/C++ language-specific code review checklist.
"""

CHECKLIST = """
## C/C++ 代码审查检查清单

### 内存管理
- [ ] 每个 `malloc`/`new` 是否有对应的 `free`/`delete`
- [ ] 是否存在内存泄漏
- [ ] 是否有缓冲区溢出风险
- [ ] 指针使用前是否检查 NULL
- [ ] 是否正确使用智能指针 (C++)

### 并发安全
- [ ] 是否正确加锁/解锁
- [ ] 是否有潜在的死锁风险
- [ ] 是否有竞态条件
- [ ] 原子操作是否正确使用

### 资源管理
- [ ] 文件句柄是否正确关闭
- [ ] 是否存在资源泄漏

### 错误处理
- [ ] 错误码是否被检查和处理
- [ ] 异常是否被正确捕获和处理 (C++)
- [ ] 是否有未处理的边界情况

### 安全性
- [ ] 是否存在整数溢出风险
- [ ] 是否存在格式化字符串漏洞
- [ ] 是否存在类型转换问题

### 性能
- [ ] 是否有不必要的拷贝
- [ ] 循环和递归是否有正确的终止条件
- [ ] 数据结构选择是否合适

### 代码质量
- [ ] 是否遵循命名规范
- [ ] 函数参数是否过多
- [ ] 函数是否过长/过于复杂
- [ ] 注释是否充分且有意义
- [ ] 是否有魔法数字/硬编码

### 编译器警告
- [ ] 是否有未解决的编译器警告
- [ ] 是否启用了合适的警告级别
"""



import os
from typing import List, Optional

from langchain.docstore.document import Document

from .embedding_manager import EmbeddingManager
from .llm_interface import JarvisPlatform_LLM, LLMInterface, ToolAgent_LLM
from .query_rewriter import QueryRewriter
from .reranker import Reranker
from .retriever import ChromaRetriever
from jarvis.jarvis_utils.config import (
    get_rag_embedding_model,
    get_rag_rerank_model,
    get_rag_vector_db_path,
    get_rag_embedding_cache_path,
)


class JarvisRAGPipeline:
    """
    RAG管道的主要协调器。

    该类集成了嵌入管理器、检索器和LLM，为添加文档和查询
    提供了一个完整的管道。
    """

    def __init__(
        self,
        llm: Optional[LLMInterface] = None,
        embedding_model: Optional[str] = None,
        db_path: Optional[str] = None,
        collection_name: str = "jarvis_rag_collection",
        use_bm25: bool = True,
        use_rerank: bool = True,
    ):
        """
        初始化RAG管道。

        参数:
            llm: 实现LLMInterface接口的类的实例。
                 如果为None，则默认为ToolAgent_LLM。
            embedding_model: 嵌入模型的名称。如果为None，则使用配置值。
            db_path: 持久化向量数据库的路径。如果为None，则使用配置值。
            collection_name: 向量数据库中集合的名称。
            use_bm25: 是否在检索中使用BM25。
            use_rerank: 是否在检索后使用重排器。
        """
        # 确定嵌入模型以隔离数据路径
        model_name = embedding_model or get_rag_embedding_model()
        sanitized_model_name = model_name.replace("/", "_").replace("\\", "_")

        # 如果给定了特定的db_path，则使用它。否则，创建一个特定于模型的路径。
        _final_db_path = (
            str(db_path)
            if db_path
            else os.path.join(get_rag_vector_db_path(), sanitized_model_name)
        )
        # 始终创建一个特定于模型的缓存路径。
        _final_cache_path = os.path.join(
            get_rag_embedding_cache_path(), sanitized_model_name
        )

        # 存储初始化参数以供延迟加载
        self.llm = llm if llm is not None else ToolAgent_LLM()
        self.embedding_model_name = embedding_model or get_rag_embedding_model()
        self.db_path = db_path
        self.collection_name = collection_name
        self.use_bm25 = use_bm25
        self.use_rerank = use_rerank

        # 延迟加载的组件
        self._embedding_manager: Optional[EmbeddingManager] = None
        self._retriever: Optional[ChromaRetriever] = None
        self._reranker: Optional[Reranker] = None
        self._query_rewriter: Optional[QueryRewriter] = None

        print("✅ JarvisRAGPipeline 初始化成功 (模型按需加载).")

    def _get_embedding_manager(self) -> EmbeddingManager:
        if self._embedding_manager is None:
            sanitized_model_name = self.embedding_model_name.replace("/", "_").replace(
                "\\", "_"
            )
            _final_cache_path = os.path.join(
                get_rag_embedding_cache_path(), sanitized_model_name
            )
            self._embedding_manager = EmbeddingManager(
                model_name=self.embedding_model_name,
                cache_dir=_final_cache_path,
            )
        return self._embedding_manager

    def _get_retriever(self) -> ChromaRetriever:
        if self._retriever is None:
            sanitized_model_name = self.embedding_model_name.replace("/", "_").replace(
                "\\", "_"
            )
            _final_db_path = (
                str(self.db_path)
                if self.db_path
                else os.path.join(get_rag_vector_db_path(), sanitized_model_name)
            )
            self._retriever = ChromaRetriever(
                embedding_manager=self._get_embedding_manager(),
                db_path=_final_db_path,
                collection_name=self.collection_name,
            )
        return self._retriever

    def _get_collection(self):
        """
        在不加载嵌入模型的情况下，直接获取并返回Chroma集合对象。
        这对于仅需要访问集合元数据（如列出文档）而无需嵌入功能的操作非常有用。
        """
        # 为了避免初始化embedding_manager，我们直接构建db_path
        if self._retriever:
            return self._retriever.collection

        sanitized_model_name = self.embedding_model_name.replace("/", "_").replace(
            "\\", "_"
        )
        _final_db_path = (
            str(self.db_path)
            if self.db_path
            else os.path.join(get_rag_vector_db_path(), sanitized_model_name)
        )

        # 直接创建ChromaRetriever所使用的chroma_client，但绕过embedding_manager
        import chromadb

        chroma_client = chromadb.PersistentClient(path=_final_db_path)
        return chroma_client.get_collection(name=self.collection_name)

    def _get_reranker(self) -> Reranker:
        if self._reranker is None:
            self._reranker = Reranker(model_name=get_rag_rerank_model())
        return self._reranker

    def _get_query_rewriter(self) -> QueryRewriter:
        if self._query_rewriter is None:
            # 使用标准LLM执行查询重写任务，而不是代理
            self._query_rewriter = QueryRewriter(JarvisPlatform_LLM())
        return self._query_rewriter

    def add_documents(self, documents: List[Document]):
        """
        将文档添加到向量知识库。

        参数:
            documents: 要添加的LangChain文档对象列表。
        """
        self._get_retriever().add_documents(documents)

    def _create_prompt(self, query: str, context_docs: List[Document]) -> str:
        """为LLM或代理创建最终的提示。"""
        context_details = []
        for doc in context_docs:
            source = doc.metadata.get("source", "未知来源")
            content = doc.page_content
            context_details.append(f"来源: {source}\n\n---\n{content}\n---")
        context = "\n\n".join(context_details)

        prompt_template = f"""
        你是一个专家助手。请根据用户的问题，结合下面提供的参考信息来回答。

        **重要**: 提供的上下文**仅供参考**，可能不完整或已过时。在回答前，你应该**优先使用工具（如 read_code）来获取最新、最准确的信息**。

        参考上下文:
        ---
        {context}
        ---

        问题: {query}

        回答:
        """
        return prompt_template.strip()

    def query(self, query_text: str, n_results: int = 5) -> str:
        """
        使用多查询检索和重排管道对知识库执行查询。

        参数:
            query_text: 用户的原始问题。
            n_results: 要检索的最终相关块的数量。

        返回:
            由LLM生成的答案。
        """
        # 1. 将原始查询重写为多个查询
        rewritten_queries = self._get_query_rewriter().rewrite(query_text)

        # 2. 为每个重写的查询检索初始候选文档
        all_candidate_docs = []
        for q in rewritten_queries:
            print(f"🔍 正在为查询变体 '{q}' 进行混合检索...")
            candidates = self._get_retriever().retrieve(
                q, n_results=n_results * 2, use_bm25=self.use_bm25
            )
            all_candidate_docs.extend(candidates)

        # 对候选文档进行去重
        unique_docs_dict = {doc.page_content: doc for doc in all_candidate_docs}
        unique_candidate_docs = list(unique_docs_dict.values())

        if not unique_candidate_docs:
            return "我在提供的文档中找不到任何相关信息来回答您的问题。"

        # 3. 根据*原始*查询对统一的候选池进行重排
        if self.use_rerank:
            print(f"🔍 正在对 {len(unique_candidate_docs)} 个候选文档进行重排（基于原始问题）...")
            retrieved_docs = self._get_reranker().rerank(
                query_text, unique_candidate_docs, top_n=n_results
            )
        else:
            retrieved_docs = unique_candidate_docs[:n_results]

        if not retrieved_docs:
            return "我在提供的文档中找不到任何相关信息来回答您的问题。"

        # 打印最终检索到的文档的来源
        sources = sorted(
            list(
                {
                    doc.metadata["source"]
                    for doc in retrieved_docs
                    if "source" in doc.metadata
                }
            )
        )
        if sources:
            print(f"📚 根据以下文档回答:")
            for source in sources:
                print(f"  - {source}")

        # 4. 创建最终提示并生成答案
        # 我们使用原始的query_text作为给LLM的最终提示
        prompt = self._create_prompt(query_text, retrieved_docs)

        print("🤖 正在从LLM生成答案...")
        answer = self.llm.generate(prompt)

        return answer

    def retrieve_only(self, query_text: str, n_results: int = 5) -> List[Document]:
        """
        仅执行检索和重排，不生成答案。

        参数:
            query_text: 用户的原始问题。
            n_results: 要检索的最终相关块的数量。

        返回:
            检索到的文档列表。
        """
        # 1. 重写查询
        rewritten_queries = self._get_query_rewriter().rewrite(query_text)

        # 2. 检索候选文档
        all_candidate_docs = []
        for q in rewritten_queries:
            print(f"🔍 正在为查询变体 '{q}' 进行混合检索...")
            candidates = self._get_retriever().retrieve(
                q, n_results=n_results * 2, use_bm25=self.use_bm25
            )
            all_candidate_docs.extend(candidates)

        unique_docs_dict = {doc.page_content: doc for doc in all_candidate_docs}
        unique_candidate_docs = list(unique_docs_dict.values())

        if not unique_candidate_docs:
            return []

        # 3. 重排
        if self.use_rerank:
            print(f"🔍 正在对 {len(unique_candidate_docs)} 个候选文档进行重排...")
            retrieved_docs = self._get_reranker().rerank(
                query_text, unique_candidate_docs, top_n=n_results
            )
        else:
            retrieved_docs = unique_candidate_docs[:n_results]

        return retrieved_docs



# -*- coding: utf-8 -*-
import os
from functools import lru_cache
from typing import Any, Dict, List, Optional

import yaml  # type: ignore

from jarvis.jarvis_utils.builtin_replace_map import BUILTIN_REPLACE_MAP

# 全局环境变量存储

GLOBAL_CONFIG_DATA: Dict[str, Any] = {}


def set_global_env_data(env_data: Dict[str, Any]) -> None:
    """设置全局环境变量数据"""
    global GLOBAL_CONFIG_DATA
    GLOBAL_CONFIG_DATA = env_data


def set_config(key: str, value: Any) -> None:
    """设置配置"""
    GLOBAL_CONFIG_DATA[key] = value


"""配置管理模块。

该模块提供了获取Jarvis系统各种配置设置的函数。
所有配置都从环境变量中读取，带有回退默认值。
"""


def get_git_commit_prompt() -> str:
    """
    获取Git提交提示模板

    返回:
        str: Git提交信息生成提示模板，如果未配置则返回空字符串
    """
    return GLOBAL_CONFIG_DATA.get("JARVIS_GIT_COMMIT_PROMPT", "")


# 输出窗口预留大小
INPUT_WINDOW_REVERSE_SIZE = 2048


@lru_cache(maxsize=None)
def get_replace_map() -> dict:
    """
    获取替换映射表。

    优先使用GLOBAL_CONFIG_DATA['JARVIS_REPLACE_MAP']的配置，
    如果没有则从数据目录下的replace_map.yaml文件中读取替换映射表，
    如果文件不存在则返回内置替换映射表。

    返回:
        dict: 合并后的替换映射表字典(内置+文件中的映射表)
    """
    if "JARVIS_REPLACE_MAP" in GLOBAL_CONFIG_DATA:
        return {**BUILTIN_REPLACE_MAP, **GLOBAL_CONFIG_DATA["JARVIS_REPLACE_MAP"]}

    replace_map_path = os.path.join(get_data_dir(), "replace_map.yaml")
    if not os.path.exists(replace_map_path):
        return BUILTIN_REPLACE_MAP.copy()

    from jarvis.jarvis_utils.output import OutputType, PrettyOutput

    PrettyOutput.print(
        "警告：使用replace_map.yaml进行配置的方式已被弃用，将在未来版本中移除。"
        "请迁移到使用GLOBAL_CONFIG_DATA中的JARVIS_REPLACE_MAP配置。",
        output_type=OutputType.WARNING,
    )

    with open(replace_map_path, "r", encoding="utf-8", errors="ignore") as file:
        file_map = yaml.safe_load(file) or {}
        return {**BUILTIN_REPLACE_MAP, **file_map}


def get_max_token_count(model_group_override: Optional[str] = None) -> int:
    """
    获取模型允许的最大token数量。

    返回:
        int: 模型能处理的最大token数量。
    """
    config = _get_resolved_model_config(model_group_override)
    return int(config.get("JARVIS_MAX_TOKEN_COUNT", "960000"))


def get_max_input_token_count(model_group_override: Optional[str] = None) -> int:
    """
    获取模型允许的最大输入token数量。

    返回:
        int: 模型能处理的最大输入token数量。
    """
    config = _get_resolved_model_config(model_group_override)
    return int(config.get("JARVIS_MAX_INPUT_TOKEN_COUNT", "32000"))


def get_shell_name() -> str:
    """
    获取系统shell名称。

    返回：
        str: Shell名称（例如bash, zsh, fish），默认为bash

    获取顺序：
    1. 先从GLOBAL_CONFIG_DATA中获取JARVIS_SHELL配置
    2. 再从GLOBAL_CONFIG_DATA中获取SHELL配置
    3. 最后从环境变量SHELL获取
    4. 如果都未配置，则默认返回bash
    """
    shell_path = GLOBAL_CONFIG_DATA.get("SHELL", os.getenv("SHELL", "/bin/bash"))
    return os.path.basename(shell_path).lower()


def _get_resolved_model_config(
    model_group_override: Optional[str] = None,
) -> Dict[str, Any]:
    """
    解析并合并模型配置，处理模型组。

    优先级顺序:
    1. 单独的环境变量 (JARVIS_PLATFORM, JARVIS_MODEL, etc.)
    2. JARVIS_LLM_GROUP 中定义的组配置
    3. 代码中的默认值

    返回:
        Dict[str, Any]: 解析后的模型配置字典
    """
    group_config = {}
    model_group_name = model_group_override or GLOBAL_CONFIG_DATA.get(
        "JARVIS_LLM_GROUP"
    )
    # The format is a list of single-key dicts: [{'group_name': {...}}, ...]
    model_groups = GLOBAL_CONFIG_DATA.get("JARVIS_LLM_GROUPS", [])

    if model_group_name and isinstance(model_groups, list):
        for group_item in model_groups:
            if isinstance(group_item, dict) and model_group_name in group_item:
                group_config = group_item[model_group_name]
                break

    # Start with group config
    resolved_config = group_config.copy()

    # Override with specific settings from GLOBAL_CONFIG_DATA
    for key in [
        "JARVIS_PLATFORM",
        "JARVIS_MODEL",
        "JARVIS_THINKING_PLATFORM",
        "JARVIS_THINKING_MODEL",
        "JARVIS_MAX_TOKEN_COUNT",
        "JARVIS_MAX_INPUT_TOKEN_COUNT",
        "JARVIS_MAX_BIG_CONTENT_SIZE",
    ]:
        if key in GLOBAL_CONFIG_DATA:
            resolved_config[key] = GLOBAL_CONFIG_DATA[key]

    return resolved_config


def get_normal_platform_name(model_group_override: Optional[str] = None) -> str:
    """
    获取正常操作的平台名称。

    返回：
        str: 平台名称，默认为'yuanbao'
    """
    config = _get_resolved_model_config(model_group_override)
    return config.get("JARVIS_PLATFORM", "yuanbao")


def get_normal_model_name(model_group_override: Optional[str] = None) -> str:
    """
    获取正常操作的模型名称。

    返回：
        str: 模型名称，默认为'deep_seek_v3'
    """
    config = _get_resolved_model_config(model_group_override)
    return config.get("JARVIS_MODEL", "deep_seek_v3")


def get_thinking_platform_name(model_group_override: Optional[str] = None) -> str:
    """
    获取思考操作的平台名称。

    返回：
        str: 平台名称，默认为正常操作平台
    """
    config = _get_resolved_model_config(model_group_override)
    # Fallback to normal platform if thinking platform is not specified
    return config.get(
        "JARVIS_THINKING_PLATFORM", get_normal_platform_name(model_group_override)
    )


def get_thinking_model_name(model_group_override: Optional[str] = None) -> str:
    """
    获取思考操作的模型名称。

    返回：
        str: 模型名称，默认为正常操作模型
    """
    config = _get_resolved_model_config(model_group_override)
    # Fallback to normal model if thinking model is not specified
    return config.get(
        "JARVIS_THINKING_MODEL", get_normal_model_name(model_group_override)
    )


def is_execute_tool_confirm() -> bool:
    """
    检查工具执行是否需要确认。

    返回：
        bool: 如果需要确认则返回True，默认为False
    """
    return GLOBAL_CONFIG_DATA.get("JARVIS_EXECUTE_TOOL_CONFIRM", False) == True


def is_confirm_before_apply_patch() -> bool:
    """
    检查应用补丁前是否需要确认。

    返回：
        bool: 如果需要确认则返回True，默认为False
    """
    return GLOBAL_CONFIG_DATA.get("JARVIS_CONFIRM_BEFORE_APPLY_PATCH", False) == True


def get_data_dir() -> str:
    """
    获取Jarvis数据存储目录路径。

    返回:
        str: 数据目录路径，优先从JARVIS_DATA_PATH环境变量获取，
             如果未设置或为空，则使用~/.jarvis作为默认值
    """
    return os.path.expanduser(
        GLOBAL_CONFIG_DATA.get("JARVIS_DATA_PATH", "~/.jarvis").strip()
    )


def get_max_big_content_size(model_group_override: Optional[str] = None) -> int:
    """
    获取最大大内容大小。

    返回：
        int: 最大大内容大小
    """
    config = _get_resolved_model_config(model_group_override)
    return int(config.get("JARVIS_MAX_BIG_CONTENT_SIZE", "160000"))


def get_pretty_output() -> bool:
    """
    获取是否启用PrettyOutput。

    返回：
        bool: 如果启用PrettyOutput则返回True，默认为True
    """
    return GLOBAL_CONFIG_DATA.get("JARVIS_PRETTY_OUTPUT", False) == True


def is_use_methodology() -> bool:
    """
    获取是否启用方法论。

    返回：
        bool: 如果启用方法论则返回True，默认为True
    """
    return GLOBAL_CONFIG_DATA.get("JARVIS_USE_METHODOLOGY", True) == True


def is_use_analysis() -> bool:
    """
    获取是否启用任务分析。

    返回：
        bool: 如果启用任务分析则返回True，默认为True
    """
    return GLOBAL_CONFIG_DATA.get("JARVIS_USE_ANALYSIS", True) == True


def get_tool_load_dirs() -> List[str]:
    """
    获取工具加载目录。

    返回:
        List[str]: 工具加载目录列表
    """
    return GLOBAL_CONFIG_DATA.get("JARVIS_TOOL_LOAD_DIRS", [])


def get_methodology_dirs() -> List[str]:
    """
    获取方法论加载目录。

    返回:
        List[str]: 方法论加载目录列表
    """
    return GLOBAL_CONFIG_DATA.get("JARVIS_METHODOLOGY_DIRS", [])


def is_print_prompt() -> bool:
    """
    获取是否打印提示。

    返回：
        bool: 如果打印提示则返回True，默认为True
    """
    return GLOBAL_CONFIG_DATA.get("JARVIS_PRINT_PROMPT", False) == True


def is_enable_static_analysis() -> bool:
    """
    获取是否启用静态代码分析。

    返回：
        bool: 如果启用静态代码分析则返回True，默认为True
    """
    return GLOBAL_CONFIG_DATA.get("JARVIS_ENABLE_STATIC_ANALYSIS", True) is True


def get_mcp_config() -> List[Dict[str, Any]]:
    """
    获取MCP配置列表。

    返回:
        List[Dict[str, Any]]: MCP配置项列表，如果未配置则返回空列表
    """
    return GLOBAL_CONFIG_DATA.get("JARVIS_MCP", [])


# ==============================================================================
# RAG Framework Configuration
# ==============================================================================


DEFAULT_RAG_GROUPS = [
    {
        "text": {
            "embedding_model": "BAAI/bge-m3",
            "rerank_model": "BAAI/bge-reranker-v2-m3",
            "use_bm25": True,
            "use_rerank": True,
        }
    },
    {
        "code": {
            "embedding_model": "Qodo/Qodo-Embed-1-7B",
            "use_bm25": False,
            "use_rerank": False,
        }
    },
]


def _get_resolved_rag_config(
    rag_group_override: Optional[str] = None,
) -> Dict[str, Any]:
    """
    解析并合并RAG配置，处理RAG组。

    优先级顺序:
    1. JARVIS_RAG 中的顶级设置 (embedding_model, etc.)
    2. JARVIS_RAG_GROUP 中定义的组配置
    3. 代码中的默认值

    返回:
        Dict[str, Any]: 解析后的RAG配置字典
    """
    group_config = {}
    rag_group_name = rag_group_override or GLOBAL_CONFIG_DATA.get("JARVIS_RAG_GROUP")
    rag_groups = GLOBAL_CONFIG_DATA.get("JARVIS_RAG_GROUPS", DEFAULT_RAG_GROUPS)

    if rag_group_name and isinstance(rag_groups, list):
        for group_item in rag_groups:
            if isinstance(group_item, dict) and rag_group_name in group_item:
                group_config = group_item[rag_group_name]
                break

    # Start with group config
    resolved_config = group_config.copy()

    # Override with specific settings from the top-level JARVIS_RAG dict
    top_level_rag_config = GLOBAL_CONFIG_DATA.get("JARVIS_RAG", {})
    if isinstance(top_level_rag_config, dict):
        for key in [
            "embedding_model",
            "rerank_model",
            "use_bm25",
            "use_rerank",
        ]:
            if key in top_level_rag_config:
                resolved_config[key] = top_level_rag_config[key]

    return resolved_config


def get_rag_embedding_model() -> str:
    """
    获取RAG嵌入模型的名称。

    返回:
        str: 嵌入模型的名称
    """
    config = _get_resolved_rag_config()
    return config.get("embedding_model", "BAAI/bge-m3")


def get_rag_rerank_model() -> str:
    """
    获取RAG rerank模型的名称。

    返回:
        str: rerank模型的名称
    """
    config = _get_resolved_rag_config()
    return config.get("rerank_model", "BAAI/bge-reranker-v2-m3")


def get_rag_embedding_cache_path() -> str:
    """
    获取RAG嵌入缓存的路径。

    返回:
        str: 缓存路径
    """
    return ".jarvis/rag/embeddings"


def get_rag_vector_db_path() -> str:
    """
    获取RAG向量数据库的路径。

    返回:
        str: 数据库路径
    """
    return ".jarvis/rag/vectordb"


def get_rag_use_bm25() -> bool:
    """
    获取RAG是否使用BM25。

    返回:
        bool: 如果使用BM25则返回True，默认为True
    """
    config = _get_resolved_rag_config()
    return config.get("use_bm25", True) is True


def get_rag_use_rerank() -> bool:
    """
    获取RAG是否使用rerank。

    返回:
        bool: 如果使用rerank则返回True，默认为True
    """
    config = _get_resolved_rag_config()
    return config.get("use_rerank", True) is True



# -*- coding: utf-8 -*-
import subprocess
from typing import Dict

import typer

from jarvis.jarvis_git_utils.git_commiter import GitCommitTool
from jarvis.jarvis_utils.output import OutputType, PrettyOutput
from jarvis.jarvis_utils.utils import init_env
from jarvis.jarvis_utils.input import user_confirm

app = typer.Typer(help="Git squash tool")


class GitSquashTool:
    name = "git_squash_agent"
    description = "Squash commits interactively using a base commit hash"
    labels = ["git", "version_control", "squash"]

    def _confirm_squash(self) -> bool:
        """Prompt user for confirmation to squash commits"""
        return user_confirm("是否确认压缩提交?", default=True)

    def _reset_to_commit(self, commit_hash: str) -> bool:
        """Perform soft reset to specified commit hash"""
        try:
            subprocess.Popen(
                ["git", "reset", "--mixed", commit_hash],
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
            ).wait()
            return True
        except Exception:
            return False

    def execute(self, args: Dict):
        """Execute the squash operation"""
        try:
            if not self._confirm_squash():
                PrettyOutput.print("操作已取消", OutputType.WARNING)
                return

            if not self._reset_to_commit(args["commit_hash"]):
                PrettyOutput.print("重置到指定提交失败", OutputType.WARNING)
                return

            # Use existing GitCommitTool for new commit
            commit_tool = GitCommitTool()
            commit_tool.execute({"lang": args.get("lang", "Chinese")})
        except Exception as e:
            PrettyOutput.print(f"压缩提交失败: {str(e)}", OutputType.WARNING)


@app.command()
def cli(
    commit_hash: str = typer.Argument(..., help="Base commit hash to squash from"),
    lang: str = typer.Option("Chinese", "--lang", help="Language for commit messages"),
):
    init_env("欢迎使用 Jarvis-GitSquash，您的Git压缩助手已准备就绪！")
    tool = GitSquashTool()
    tool.execute({"commit_hash": commit_hash, "lang": lang})


def main():
    """Application entry point"""
    app()


if __name__ == "__main__":
    main()



import hashlib
from typing import List, Optional, Any

from diskcache import Cache


class EmbeddingCache:
    """
    一个用于存储和检索文本嵌入的基于磁盘的缓存。

    该类使用diskcache创建一个持久化的本地缓存。它根据每个文本内容的
    SHA256哈希值为其生成一个键，使得查找过程具有确定性和高效性。
    """

    def __init__(self, cache_dir: str, salt: str = ""):
        """
        初始化EmbeddingCache。

        参数:
            cache_dir (str): 缓存将要存储的目录。
            salt (str): 添加到哈希中的盐值。这对于确保由不同模型生成的
                        嵌入不会发生冲突至关重要。例如，可以使用模型名称作为盐值。
        """
        self.cache = Cache(cache_dir)
        self.salt = salt

    def _get_key(self, text: str) -> str:
        """为一个给定的文本和盐值生成一个唯一的缓存键。"""
        hash_object = hashlib.sha256((self.salt + text).encode("utf-8"))
        return hash_object.hexdigest()

    def get(self, text: str) -> Optional[Any]:
        """
        从缓存中检索一个嵌入。

        参数:
            text (str): 要查找的文本。

        返回:
            缓存的嵌入，如果不在缓存中则返回None。
        """
        key = self._get_key(text)
        return self.cache.get(key)

    def set(self, text: str, embedding: Any) -> None:
        """
        在缓存中存储一个嵌入。

        参数:
            text (str): 与嵌入相对应的文本。
            embedding (Any): 要存储的嵌入向量。
        """
        key = self._get_key(text)
        self.cache.set(key, embedding)

    def get_batch(self, texts: List[str]) -> List[Optional[Any]]:
        """
        从缓存中检索一批嵌入。

        参数:
            texts (List[str]): 要查找的文本列表。

        返回:
            一个列表，其中包含缓存的嵌入，对于缓存未命中的情况则为None。
        """
        return [self.get(text) for text in texts]

    def set_batch(self, texts: List[str], embeddings: List[Any]) -> None:
        """
        在缓存中存储一批嵌入。

        参数:
            texts (List[str]): 文本列表。
            embeddings (List[Any]): 相应的嵌入列表。
        """
        if len(texts) != len(embeddings):
            raise ValueError("Length of texts and embeddings must be the same.")

        with self.cache.transact():
            for text, embedding in zip(texts, embeddings):
                self.set(text, embedding)

    def close(self):
        """关闭缓存连接。"""
        self.cache.close()



# -*- coding: utf-8 -*-

import requests
from typing import Any, Dict, Optional, Generator


def get_requests_session() -> requests.Session:
    """
    获取一个配置好的 requests.Session 对象

    返回:
        requests.Session 对象
    """
    session = requests.Session()
    session.headers.update(
        {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/136.0.0.0 Safari/537.36",
        }
    )
    return session


# 增强版本的 HTTP 请求方法（使用 requests 实现）
def post(
    url: str,
    data: Optional[Any] = None,
    json: Optional[Dict[str, Any]] = None,
    **kwargs,
) -> requests.Response:
    """
    发送增强版永不超时的 POST 请求，使用 requests 实现

    参数:
        url: 请求的 URL
        data: (可选) 请求体数据 (表单数据或原始数据)
        json: (可选) JSON 数据，会自动设置 Content-Type
        **kwargs: 其他传递给 requests.post 的参数

    返回:
        requests.Response 对象

    注意:
        此方法使用 requests 实现。要实现重试，请考虑使用 Session 和 HTTPAdapter。
        永不超时通过 timeout=None 设置。
    """
    kwargs.setdefault("timeout", None)
    with get_requests_session() as session:
        response = session.post(url=url, data=data, json=json, **kwargs)
        response.raise_for_status()
        return response


def get(url: str, **kwargs) -> requests.Response:
    """
    发送增强版永不超时的 GET 请求，使用 requests 实现

    参数:
        url: 请求的 URL
        **kwargs: 其他传递给 requests.get 的参数

    返回:
        requests.Response 对象

    注意:
        此方法使用 requests 实现。
        永不超时通过 timeout=None 设置。
    """
    kwargs.setdefault("timeout", None)
    with get_requests_session() as session:
        response = session.get(url=url, **kwargs)
        response.raise_for_status()
        return response


def put(url: str, data: Optional[Any] = None, **kwargs) -> requests.Response:
    """
    发送增强版永不超时的 PUT 请求，使用 requests 实现

    参数:
        url: 请求的 URL
        data: (可选) 请求体数据 (表单数据或原始数据)
        **kwargs: 其他传递给 requests.put 的参数

    返回:
        requests.Response 对象

    注意:
        此方法使用 requests 实现。
        永不超时通过 timeout=None 设置。
    """
    kwargs.setdefault("timeout", None)
    with get_requests_session() as session:
        response = session.put(url=url, data=data, **kwargs)
        response.raise_for_status()
        return response


def delete(url: str, **kwargs) -> requests.Response:
    """
    发送增强版永不超时的 DELETE 请求，使用 requests 实现

    参数:
        url: 请求的 URL
        **kwargs: 其他传递给 requests.delete 的参数

    返回:
        requests.Response 对象

    注意:
        此方法使用 requests 实现。
        永不超时通过 timeout=None 设置。
    """
    kwargs.setdefault("timeout", None)
    with get_requests_session() as session:
        response = session.delete(url=url, **kwargs)
        response.raise_for_status()
        return response


# 同步流式POST请求方法
def stream_post(
    url: str,
    data: Optional[Any] = None,
    json: Optional[Dict[str, Any]] = None,
    **kwargs,
) -> Generator[str, None, None]:
    """
    发送流式 POST 请求，使用 requests 实现，返回解码后的字符串行生成器

    参数:
        url: 请求的 URL
        data: (可选) 请求体数据 (表单数据或原始数据)
        json: (可选) JSON 数据，会自动设置 Content-Type
        **kwargs: 其他传递给 requests.post 的参数

    返回:
        Generator[str, None, None]: 字符串行生成器

    注意:
        此方法使用 requests 实现流式请求，适用于处理大文件下载或流式响应
    """
    kwargs.setdefault("timeout", None)
    with get_requests_session() as session:
        with session.post(url, data=data, json=json, stream=True, **kwargs) as response:
            response.raise_for_status()
            for line in response.iter_lines(chunk_size=1):
                if line:
                    yield line.decode("utf-8", errors="ignore")



# -*- coding: utf-8 -*-
import json
import threading
from typing import Any, Callable, Dict, List
from urllib.parse import urljoin

import requests

from jarvis.jarvis_mcp import McpClient
from jarvis.jarvis_utils.output import OutputType, PrettyOutput


class StreamableMcpClient(McpClient):
    """Streamable HTTP MCP客户端实现

    参数:
        config: 配置字典，包含以下字段：
            - base_url: str - MCP服务器的基础URL
            - auth_token: str - 认证令牌（可选）
            - headers: Dict[str, str] - 额外的HTTP头（可选）
    """

    def __init__(self, config: Dict[str, Any]):
        self.config = config
        self.base_url = config.get("base_url", "")
        if not self.base_url:
            raise ValueError("No base_url specified in config")

        # 设置HTTP客户端
        self.session = requests.Session()
        self.session.headers.update(
            {
                "Content-Type": "application/json",
                "Accept": "application/json",
            }
        )

        # 添加认证令牌（如果提供）
        auth_token = config.get("auth_token")
        if auth_token:
            self.session.headers["Authorization"] = f"Bearer {auth_token}"

        # 添加额外的HTTP头
        extra_headers = config.get("headers", {})
        self.session.headers.update(extra_headers)

        # 请求相关属性
        self.pending_requests = {}  # 存储等待响应的请求 {id: Event}
        self.request_results = {}  # 存储请求结果 {id: result}
        self.notification_handlers = {}
        self.event_lock = threading.Lock()
        self.request_id_counter = 0

        # 初始化连接
        self._initialize()

    def _initialize(self) -> None:
        """初始化MCP连接"""
        try:
            # 发送初始化请求
            response = self._send_request(
                "initialize",
                {
                    "processId": None,  # 远程客户端不需要进程ID
                    "clientInfo": {"name": "jarvis", "version": "1.0.0"},
                    "capabilities": {},
                    "protocolVersion": "2025-03-26",
                },
            )

            # 验证服务器响应
            if "result" not in response:
                raise RuntimeError(
                    f"初始化失败: {response.get('error', 'Unknown error')}"
                )

            # 发送initialized通知
            self._send_notification("notifications/initialized", {})

        except Exception as e:
            PrettyOutput.print(f"MCP初始化失败: {str(e)}", OutputType.ERROR)
            raise

    def register_notification_handler(self, method: str, handler: Callable) -> None:
        """注册通知处理器

        参数:
            method: 通知方法名
            handler: 处理通知的回调函数，接收params参数
        """
        with self.event_lock:
            if method not in self.notification_handlers:
                self.notification_handlers[method] = []
            self.notification_handlers[method].append(handler)

    def unregister_notification_handler(self, method: str, handler: Callable) -> None:
        """注销通知处理器

        参数:
            method: 通知方法名
            handler: 要注销的处理器函数
        """
        with self.event_lock:
            if method in self.notification_handlers:
                if handler in self.notification_handlers[method]:
                    self.notification_handlers[method].remove(handler)
                if not self.notification_handlers[method]:
                    del self.notification_handlers[method]

    def _get_next_request_id(self) -> str:
        """获取下一个请求ID"""
        with self.event_lock:
            self.request_id_counter += 1
            return str(self.request_id_counter)

    def _send_request(self, method: str, params: Dict[str, Any]) -> Dict[str, Any]:
        """发送请求到MCP服务器

        参数:
            method: 请求方法
            params: 请求参数

        返回:
            Dict[str, Any]: 响应结果
        """
        # 生成唯一请求ID
        req_id = self._get_next_request_id()

        # 创建事件标志，用于等待响应
        event = threading.Event()

        with self.event_lock:
            self.pending_requests[req_id] = event

        try:
            # 构建请求
            request = {
                "jsonrpc": "2.0",
                "method": method,
                "params": params,
                "id": req_id,
            }

            # 发送请求到Streamable HTTP端点
            mcp_url = urljoin(self.base_url, "mcp")
            response = self.session.post(
                mcp_url, json=request, stream=True  # 启用流式传输
            )
            response.raise_for_status()

            # 处理流式响应
            result = None
            for line in response.iter_lines(decode_unicode=True):
                if line:
                    try:
                        data = json.loads(line)
                        if "id" in data and data["id"] == req_id:
                            # 这是我们的请求响应
                            result = data
                            break
                        elif "method" in data:
                            # 这是一个通知
                            method = data.get("method", "")
                            params = data.get("params", {})
                            if method in self.notification_handlers:
                                for handler in self.notification_handlers[method]:
                                    try:
                                        handler(params)
                                    except Exception as e:
                                        PrettyOutput.print(
                                            f"处理通知时出错 ({method}): {e}",
                                            OutputType.ERROR,
                                        )
                    except json.JSONDecodeError:
                        PrettyOutput.print(f"无法解析响应: {line}", OutputType.WARNING)
                        continue

            if result is None:
                raise RuntimeError(f"未收到响应: {method}")

            return result

        except Exception as e:
            PrettyOutput.print(f"发送请求失败: {str(e)}", OutputType.ERROR)
            raise
        finally:
            # 清理请求状态
            with self.event_lock:
                self.pending_requests.pop(req_id, None)
                self.request_results.pop(req_id, None)

    def _send_notification(self, method: str, params: Dict[str, Any]) -> None:
        """发送通知到MCP服务器（不需要响应）

        参数:
            method: 通知方法
            params: 通知参数
        """
        try:
            # 构建通知
            notification = {"jsonrpc": "2.0", "method": method, "params": params}

            # 发送通知到Streamable HTTP端点
            mcp_url = urljoin(self.base_url, "mcp")
            response = self.session.post(mcp_url, json=notification)
            response.raise_for_status()

        except Exception as e:
            PrettyOutput.print(f"发送通知失败: {str(e)}", OutputType.ERROR)
            raise

    def get_tool_list(self) -> List[Dict[str, Any]]:
        """获取工具列表

        返回:
            List[Dict[str, Any]]: 工具列表，每个工具包含以下字段：
                - name: str - 工具名称
                - description: str - 工具描述
                - parameters: Dict - 工具参数
        """
        try:
            response = self._send_request("tools/list", {})
            if "result" in response and "tools" in response["result"]:
                # 注意这里: 响应结构是 response['result']['tools']
                tools = response["result"]["tools"]
                # 将MCP协议字段转换为内部格式
                formatted_tools = []
                for tool in tools:
                    # 从inputSchema中提取参数定义
                    input_schema = tool.get("inputSchema", {})
                    parameters = {}
                    if "properties" in input_schema:
                        parameters = input_schema["properties"]

                    formatted_tools.append(
                        {
                            "name": tool.get("name", ""),
                            "description": tool.get("description", ""),
                            "parameters": parameters,
                        }
                    )
                return formatted_tools
            else:
                error_msg = "获取工具列表失败"
                if "error" in response:
                    error_msg += f": {response['error']}"
                elif "result" in response:
                    error_msg += f": 响应格式不正确 - {response['result']}"
                else:
                    error_msg += ": 未知错误"

                PrettyOutput.print(error_msg, OutputType.ERROR)
                return []
        except Exception as e:
            PrettyOutput.print(f"获取工具列表失败: {str(e)}", OutputType.ERROR)
            return []

    def execute(self, tool_name: str, arguments: Dict[str, Any]) -> Dict[str, Any]:
        """执行工具

        参数:
            tool_name: 工具名称
            arguments: 参数字典，包含工具执行所需的参数

        返回:
            Dict[str, Any]: 执行结果，包含以下字段：
                - success: bool - 是否执行成功
                - stdout: str - 标准输出
                - stderr: str - 标准错误
        """
        try:
            response = self._send_request(
                "tools/call", {"name": tool_name, "arguments": arguments}
            )
            if "result" in response:
                result = response["result"]
                # 从content中提取输出信息
                stdout = ""
                stderr = ""
                for content in result.get("content", []):
                    if content.get("type") == "text":
                        stdout += content.get("text", "")
                    elif content.get("type") == "error":
                        stderr += content.get("text", "")

                return {"success": True, "stdout": stdout, "stderr": stderr}
            else:
                return {
                    "success": False,
                    "stdout": "",
                    "stderr": response.get("error", "Unknown error"),
                }
        except Exception as e:
            PrettyOutput.print(f"执行工具失败: {str(e)}", OutputType.ERROR)
            return {"success": False, "stdout": "", "stderr": str(e)}

    def get_resource_list(self) -> List[Dict[str, Any]]:
        """获取资源列表

        返回:
            List[Dict[str, Any]]: 资源列表，每个资源包含以下字段：
                - uri: str - 资源的唯一标识符
                - name: str - 资源的名称
                - description: str - 资源的描述（可选）
                - mimeType: str - 资源的MIME类型（可选）
        """
        try:
            response = self._send_request("resources/list", {})
            if "result" in response and "resources" in response["result"]:
                return response["result"]["resources"]
            else:
                error_msg = "获取资源列表失败"
                if "error" in response:
                    error_msg += f": {response['error']}"
                else:
                    error_msg += ": 未知错误"
                PrettyOutput.print(error_msg, OutputType.ERROR)
                return []
        except Exception as e:
            PrettyOutput.print(f"获取资源列表失败: {str(e)}", OutputType.ERROR)
            return []

    def get_resource(self, uri: str) -> Dict[str, Any]:
        """获取指定资源的内容

        参数:
            uri: str - 资源的URI标识符

        返回:
            Dict[str, Any]: 执行结果，包含以下字段：
                - success: bool - 是否执行成功
                - stdout: str - 资源内容（文本或base64编码的二进制内容）
                - stderr: str - 错误信息
        """
        try:
            response = self._send_request("resources/read", {"uri": uri})
            if "result" in response and "contents" in response["result"]:
                contents = response["result"]["contents"]
                if contents:
                    content = contents[0]  # 获取第一个资源内容
                    # 根据资源类型返回内容
                    if "text" in content:
                        return {
                            "success": True,
                            "stdout": content["text"],
                            "stderr": "",
                        }
                    elif "blob" in content:
                        return {
                            "success": True,
                            "stdout": content["blob"],
                            "stderr": "",
                        }
                return {"success": False, "stdout": "", "stderr": "资源内容为空"}
            else:
                error_msg = "获取资源内容失败"
                if "error" in response:
                    error_msg += f": {response['error']}"
                else:
                    error_msg += ": 未知错误"
                PrettyOutput.print(error_msg, OutputType.ERROR)
                return {"success": False, "stdout": "", "stderr": error_msg}
        except Exception as e:
            error_msg = f"获取资源内容失败: {str(e)}"
            PrettyOutput.print(error_msg, OutputType.ERROR)
            return {"success": False, "stdout": "", "stderr": error_msg}

    def __del__(self):
        """清理资源"""
        # 清理请求状态
        with self.event_lock:
            for event in self.pending_requests.values():
                event.set()  # 释放所有等待的请求
            self.pending_requests.clear()
            self.request_results.clear()

        # 关闭HTTP会话
        if self.session:
            self.session.close()



# -*- coding: utf-8 -*-
import os
import shutil
import subprocess
import sys
from pathlib import Path
from typing import Dict, Optional

import typer
import yaml  # type: ignore
from prompt_toolkit import prompt  # type: ignore

from jarvis.jarvis_agent import (
    Agent,
    OutputType,
    PrettyOutput,
    get_multiline_input,
    origin_agent_system_prompt,
    user_confirm,
)
from jarvis.jarvis_agent.builtin_input_handler import builtin_input_handler
from jarvis.jarvis_agent.shell_input_handler import shell_input_handler
from jarvis.jarvis_tools.registry import ToolRegistry
from jarvis.jarvis_utils.config import get_data_dir
from jarvis.jarvis_utils.utils import init_env

app = typer.Typer(help="Jarvis AI assistant")


def _load_tasks() -> Dict[str, str]:
    """Load tasks from .jarvis files in user home and current directory."""
    tasks: Dict[str, str] = {}

    # Check pre-command in data directory
    data_dir = get_data_dir()
    pre_command_path = os.path.join(data_dir, "pre-command")
    if os.path.exists(pre_command_path):
        print(f"🔍 从{pre_command_path}加载预定义任务...")
        try:
            with open(pre_command_path, "r", encoding="utf-8", errors="ignore") as f:
                user_tasks = yaml.safe_load(f)
            if isinstance(user_tasks, dict):
                for name, desc in user_tasks.items():
                    if desc:
                        tasks[str(name)] = str(desc)
            print(f"✅ 预定义任务加载完成 {pre_command_path}")
        except (yaml.YAMLError, OSError):
            print(f"❌ 预定义任务加载失败 {pre_command_path}")

    # Check .jarvis/pre-command in current directory
    pre_command_path = ".jarvis/pre-command"
    if os.path.exists(pre_command_path):
        abs_path = os.path.abspath(pre_command_path)
        print(f"🔍 从{abs_path}加载预定义任务...")
        try:
            with open(pre_command_path, "r", encoding="utf-8", errors="ignore") as f:
                local_tasks = yaml.safe_load(f)
            if isinstance(local_tasks, dict):
                for name, desc in local_tasks.items():
                    if desc:
                        tasks[str(name)] = str(desc)
            print(f"✅ 预定义任务加载完成 {pre_command_path}")
        except (yaml.YAMLError, OSError):
            print(f"❌ 预定义任务加载失败 {pre_command_path}")

    return tasks


def _select_task(tasks: Dict[str, str]) -> str:
    """Let user select a task from the list or skip. Returns task description if selected."""
    if not tasks:
        return ""

    task_names = list(tasks.keys())
    task_list = ["可用任务:"]
    for i, name in enumerate(task_names, 1):
        task_list.append(f"[{i}] {name}")
    task_list.append("[0] 跳过预定义任务")
    PrettyOutput.print("\n".join(task_list), OutputType.INFO)

    while True:
        try:
            choice_str = prompt("\n请选择一个任务编号（0 跳过预定义任务）：").strip()
            if not choice_str:
                return ""

            choice = int(choice_str)
            if choice == 0:
                return ""
            if 1 <= choice <= len(task_names):
                selected_task = tasks[task_names[choice - 1]]
                PrettyOutput.print(f"将要执行任务:\n {selected_task}", OutputType.INFO)
                # 询问是否需要补充信息
                need_additional = user_confirm(
                    "需要为此任务添加补充信息吗？", default=False
                )
                if need_additional:
                    additional_input = get_multiline_input("请输入补充信息：")
                    if additional_input:
                        selected_task = (
                            f"{selected_task}\n\n补充信息:\n{additional_input}"
                        )
                return selected_task
            PrettyOutput.print(
                "无效的选择。请选择列表中的一个号码。", OutputType.WARNING
            )

        except (KeyboardInterrupt, EOFError):
            return ""
        except ValueError as val_err:
            PrettyOutput.print(f"选择任务失败: {str(val_err)}", OutputType.ERROR)


def _handle_edit_mode(edit: bool, config_file: Optional[str]) -> None:
    """If edit flag is set, open config file in editor and exit."""
    if not edit:
        return

    config_file_path = (
        Path(config_file)
        if config_file
        else Path(os.path.expanduser("~/.jarvis/config.yaml"))
    )
    editors = ["nvim", "vim", "vi"]
    editor = next((e for e in editors if shutil.which(e)), None)

    if editor:
        try:
            subprocess.run([editor, str(config_file_path)], check=True)
            raise typer.Exit(code=0)
        except (subprocess.CalledProcessError, FileNotFoundError) as e:
            PrettyOutput.print(f"Failed to open editor: {e}", OutputType.ERROR)
            raise typer.Exit(code=1)
    else:
        PrettyOutput.print(
            "No suitable editor found (nvim, vim, vi).", OutputType.ERROR
        )
        raise typer.Exit(code=1)


def _initialize_agent(
    llm_type: str, model_group: Optional[str], restore_session: bool
) -> Agent:
    """Initialize the agent and restore session if requested."""
    agent = Agent(
        system_prompt=origin_agent_system_prompt,
        llm_type=llm_type,
        model_group=model_group,
        input_handler=[shell_input_handler, builtin_input_handler],
        output_handler=[ToolRegistry()],  # type: ignore
        need_summary=False,
    )

    # 尝试恢复会话
    if restore_session:
        if agent.restore_session():
            PrettyOutput.print("会话已成功恢复。", OutputType.SUCCESS)
        else:
            PrettyOutput.print("无法恢复会话。", OutputType.WARNING)
    return agent


def _get_and_run_task(agent: Agent, task_content: Optional[str] = None) -> None:
    """Get task from various sources and run it."""
    # 优先处理命令行直接传入的任务
    if task_content:
        agent.run(task_content)
        raise typer.Exit(code=0)

    if agent.first:
        tasks = _load_tasks()
        if tasks and (selected_task := _select_task(tasks)):
            PrettyOutput.print(f"开始执行任务: \n{selected_task}", OutputType.INFO)
            agent.run(selected_task)
            raise typer.Exit(code=0)

    user_input = get_multiline_input("请输入你的任务（输入空行退出）:")
    if user_input:
        agent.run(user_input)
    raise typer.Exit(code=0)


@app.callback(invoke_without_command=True)
def run_cli(
    ctx: typer.Context,
    llm_type: str = typer.Option(
        "normal",
        "--llm_type",
        help="LLM type to use, choices are 'normal' and 'thinking'",
    ),
    task: Optional[str] = typer.Option(
        None, "-t", "--task", help="Directly input task content from command line"
    ),
    model_group: Optional[str] = typer.Option(
        None, "--model_group", help="Model group to use, overriding config"
    ),
    config_file: Optional[str] = typer.Option(
        None, "-f", "--config", help="Path to custom config file"
    ),
    restore_session: bool = typer.Option(
        False,
        "--restore-session",
        help="Restore session from .jarvis/saved_session.json",
    ),
    edit: bool = typer.Option(
        False, "-e", "--edit", help="Edit the configuration file"
    ),
) -> None:
    """Jarvis AI assistant command-line interface."""
    if ctx.invoked_subcommand is not None:
        return
        
    _handle_edit_mode(edit, config_file)

    init_env("欢迎使用 Jarvis AI 助手，您的智能助理已准备就绪！", config_file=config_file)

    try:
        agent = _initialize_agent(llm_type, model_group, restore_session)
        _get_and_run_task(agent, task)
    except typer.Exit:
        raise
    except Exception as err:  # pylint: disable=broad-except
        PrettyOutput.print(f"初始化错误: {str(err)}", OutputType.ERROR)
        raise typer.Exit(code=1)


def main() -> None:
    """Application entry point."""
    app()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
JavaScript/TypeScript language-specific code review checklist.
"""

CHECKLIST = """
## JavaScript/TypeScript 代码审查检查清单

### 代码风格和格式
- [ ] 是否遵循一致的命名约定 (camelCase、PascalCase)
- [ ] 是否使用 ESLint/TSLint 进行代码规范检查
- [ ] 缩进和格式是否一致
- [ ] 是否避免了过长的函数和过深的嵌套

### 类型安全 (TypeScript)
- [ ] 是否使用了适当的类型注解
- [ ] 是否避免了过度使用 any 类型
- [ ] 是否正确使用接口和类型别名
- [ ] 泛型使用是否合理
- [ ] 是否正确处理可能为 null/undefined 的值

### 异步编程
- [ ] Promise 异常是否被正确捕获和处理
- [ ] async/await 是否使用得当
- [ ] 是否避免了回调地狱
- [ ] 是否正确处理异步操作的竞态条件
- [ ] 是否合理使用 Promise.all/Promise.race

### 性能优化
- [ ] 是否避免了不必要的重新渲染
- [ ] 是否避免了内存泄漏 (闭包、事件监听器等)
- [ ] 是否适当使用了记忆化和缓存
- [ ] 循环和数据处理是否高效
- [ ] 是否避免了过度的DOM操作

### 安全性
- [ ] 是否防范了 XSS 攻击
- [ ] 是否防范了 CSRF 攻击
- [ ] 敏感数据是否安全处理
- [ ] 是否正确验证用户输入
- [ ] 是否避免使用 eval 和不安全的 DOM API

### 浏览器兼容性
- [ ] 是否考虑了目标浏览器的支持程度
- [ ] 是否使用了适当的垫片 (polyfills)
- [ ] 是否测试了不同浏览器下的表现

### 模块化和依赖管理
- [ ] 模块划分是否合理
- [ ] 是否避免了循环依赖
- [ ] 依赖版本是否固定
- [ ] 第三方库使用是否恰当
- [ ] 是否避免了过度依赖

### 测试
- [ ] 是否有单元测试覆盖核心功能
- [ ] 是否测试了组件渲染和交互
- [ ] 测试用例是否覆盖边界条件
- [ ] 是否有集成测试和端到端测试

### 可访问性 (对于前端代码)
- [ ] 是否遵循 ARIA 最佳实践
- [ ] 是否使用语义化 HTML
- [ ] 是否支持键盘导航
- [ ] 颜色对比度是否符合标准
- [ ] 是否兼容屏幕阅读器

### 可维护性
- [ ] 代码结构是否清晰
- [ ] 是否有充分的注释和文档
- [ ] 复杂逻辑是否抽象成可测试的函数
- [ ] 是否遵循 DRY 原则
- [ ] 状态管理是否合理
"""



import os
import pickle
from typing import Any, Dict, List, cast

import chromadb
from langchain.docstore.document import Document
from langchain.text_splitter import RecursiveCharacterTextSplitter
from rank_bm25 import BM25Okapi  # type: ignore

from .embedding_manager import EmbeddingManager


class ChromaRetriever:
    """
    一个检索器类，它结合了密集向量搜索（ChromaDB）和稀疏关键字搜索（BM25）
    以实现混合检索。
    """

    def __init__(
        self,
        embedding_manager: EmbeddingManager,
        db_path: str,
        collection_name: str = "jarvis_rag_collection",
    ):
        """
        初始化ChromaRetriever。

        参数:
            embedding_manager: EmbeddingManager的实例。
            db_path: ChromaDB持久化存储的文件路径。
            collection_name: ChromaDB中集合的名称。
        """
        self.embedding_manager = embedding_manager
        self.db_path = db_path
        self.collection_name = collection_name

        # 初始化ChromaDB客户端
        self.client = chromadb.PersistentClient(path=self.db_path)
        self.collection = self.client.get_or_create_collection(
            name=self.collection_name
        )
        print(f"✅ ChromaDB 客户端已在 '{db_path}' 初始化，集合为 '{collection_name}'。")

        # BM25索引设置
        self.bm25_index_path = os.path.join(self.db_path, f"{collection_name}_bm25.pkl")
        self._load_or_initialize_bm25()

    def _load_or_initialize_bm25(self):
        """从磁盘加载BM25索引或初始化一个新索引。"""
        if os.path.exists(self.bm25_index_path):
            print("🔍 正在加载现有的 BM25 索引...")
            with open(self.bm25_index_path, "rb") as f:
                data = pickle.load(f)
                self.bm25_corpus = data["corpus"]
                self.bm25_index = BM25Okapi(self.bm25_corpus)
            print("✅ BM25 索引加载成功。")
        else:
            print("⚠️ 未找到 BM25 索引，将初始化一个新的。")
            self.bm25_corpus = []
            self.bm25_index = None

    def _save_bm25_index(self):
        """将BM25索引保存到磁盘。"""
        if self.bm25_index:
            print("💾 正在保存 BM25 索引...")
            with open(self.bm25_index_path, "wb") as f:
                pickle.dump({"corpus": self.bm25_corpus, "index": self.bm25_index}, f)
            print("✅ BM25 索引保存成功。")

    def add_documents(
        self, documents: List[Document], chunk_size=1000, chunk_overlap=100
    ):
        """
        将文档拆分、嵌入，并添加到ChromaDB和BM25索引中。
        """
        text_splitter = RecursiveCharacterTextSplitter(
            chunk_size=chunk_size, chunk_overlap=chunk_overlap
        )
        chunks = text_splitter.split_documents(documents)

        print(f"📄 已将 {len(documents)} 个文档拆分为 {len(chunks)} 个块。")

        if not chunks:
            return

        # 提取内容、元数据并生成ID
        chunk_texts = [chunk.page_content for chunk in chunks]
        metadatas = [chunk.metadata for chunk in chunks]
        start_id = self.collection.count()
        ids = [f"doc_{i}" for i in range(start_id, start_id + len(chunks))]

        # 添加到ChromaDB
        embeddings = self.embedding_manager.embed_documents(chunk_texts)
        self.collection.add(
            ids=ids,
            embeddings=cast(Any, embeddings),
            documents=chunk_texts,
            metadatas=cast(Any, metadatas),
        )
        print(f"✅ 成功将 {len(chunks)} 个块添加到 ChromaDB 集合中。")

        # 更新并保存BM25索引
        tokenized_chunks = [doc.split() for doc in chunk_texts]
        self.bm25_corpus.extend(tokenized_chunks)
        self.bm25_index = BM25Okapi(self.bm25_corpus)
        self._save_bm25_index()

    def retrieve(
        self, query: str, n_results: int = 5, use_bm25: bool = True
    ) -> List[Document]:
        """
        使用向量搜索和BM25执行混合检索，然后使用倒数排序融合（RRF）
        对结果进行融合。
        """
        # 1. 向量搜索 (ChromaDB)
        query_embedding = self.embedding_manager.embed_query(query)
        vector_results = self.collection.query(
            query_embeddings=cast(Any, [query_embedding]),
            n_results=n_results * 2,  # 检索更多结果用于融合
        )

        # 2. 关键字搜索 (BM25)
        bm25_docs = []
        if self.bm25_index and use_bm25:
            tokenized_query = query.split()
            doc_scores = self.bm25_index.get_scores(tokenized_query)

            # 从Chroma获取所有文档以匹配BM25分数
            all_docs_in_collection = self.collection.get()
            all_documents = all_docs_in_collection.get("documents")
            all_metadatas = all_docs_in_collection.get("metadatas")

            bm25_results_with_docs = []
            if all_documents and all_metadatas:
                # 创建从索引到文档的映射
                bm25_results_with_docs = [
                    (
                        all_documents[i],
                        all_metadatas[i],
                        score,
                    )
                    for i, score in enumerate(doc_scores)
                    if score > 0
                ]

            # 按分数排序并取最高结果
            bm25_results_with_docs.sort(key=lambda x: x[2], reverse=True)

            for doc_text, metadata, _ in bm25_results_with_docs[: n_results * 2]:
                bm25_docs.append(Document(page_content=doc_text, metadata=metadata))

        # 3. 倒数排序融合 (RRF)
        fused_scores: Dict[str, float] = {}
        k = 60  # RRF排名常数

        # 处理向量结果
        if vector_results and vector_results["ids"] and vector_results["documents"]:
            vec_ids = vector_results["ids"][0]
            vec_texts = vector_results["documents"][0]

            for rank, doc_id in enumerate(vec_ids):
                fused_scores[doc_id] = fused_scores.get(doc_id, 0) + 1 / (k + rank)

            # 为BM25融合创建从文档文本到其ID的映射
            doc_text_to_id = {text: doc_id for text, doc_id in zip(vec_texts, vec_ids)}

            for rank, doc in enumerate(bm25_docs):
                bm25_doc_id = doc_text_to_id.get(doc.page_content)
                if bm25_doc_id:
                    fused_scores[bm25_doc_id] = fused_scores.get(bm25_doc_id, 0) + 1 / (
                        k + rank
                    )

        # 对融合结果进行排序
        sorted_fused_results = sorted(
            fused_scores.items(), key=lambda x: x[1], reverse=True
        )

        # 根据融合排名从ChromaDB获取最终文档
        final_doc_ids = [item[0] for item in sorted_fused_results[:n_results]]

        if not final_doc_ids:
            return []

        final_docs_data = self.collection.get(ids=final_doc_ids)

        retrieved_docs = []
        if final_docs_data:
            final_documents = final_docs_data.get("documents")
            final_metadatas = final_docs_data.get("metadatas")

            if final_documents and final_metadatas:
                for doc_text, metadata in zip(final_documents, final_metadatas):
                    if doc_text is not None and metadata is not None:
                        retrieved_docs.append(
                            Document(
                                page_content=cast(str, doc_text), metadata=metadata
                            )
                        )

        return retrieved_docs



# -*- coding: utf-8 -*-
import os
from typing import Any, Dict


from jarvis.jarvis_platform.registry import PlatformRegistry
from jarvis.jarvis_utils.output import OutputType, PrettyOutput


class FileAnalyzerTool:
    name = "file_analyzer"
    description = (
        """分析文件内容并提取关键信息。支持的文件：文本文件、word文档、pdf文件、图片"""
    )
    parameters = {
        "type": "object",
        "properties": {
            "file_paths": {
                "type": "array",
                "items": {"type": "string"},
                "description": "要分析的文件路径列表",
            },
            "prompt": {
                "type": "string",
                "description": "分析文件的提示词，指导模型提取什么样的信息",
            },
        },
        "required": ["file_paths", "prompt"],
    }

    @staticmethod
    def check() -> bool:
        return PlatformRegistry().get_thinking_platform().support_upload_files()

    def execute(self, args: Dict[str, Any]) -> Dict[str, Any]:
        """执行文件分析操作

        Args:
            args: 包含文件路径列表和提示词的字典

        Returns:
            Dict: 包含分析结果的字典
        """
        try:
            file_paths = args["file_paths"]
            prompt = args["prompt"]

            # 验证文件路径
            valid_files = []
            for file_path in file_paths:
                if os.path.exists(file_path):
                    valid_files.append(file_path)
                else:
                    PrettyOutput.print(f"文件不存在: {file_path}", OutputType.WARNING)

            if not valid_files:
                return {"success": False, "stdout": "", "stderr": "没有找到有效的文件"}

            # 创建thinking平台实例
            platform = PlatformRegistry().get_thinking_platform()

            if not platform:
                return {
                    "success": False,
                    "stdout": "",
                    "stderr": "无法创建thinking平台实例",
                }

            # 设置系统消息
            system_message = """你是一个文件分析助手。你的任务是分析提供的文件内容，并根据用户的提示提取关键信息。
请保持客观，只关注文件中实际存在的内容。如果无法确定某些信息，请明确指出。
请以结构化的方式组织你的回答，使用标题、列表和代码块等格式来提高可读性。"""
            platform.set_system_prompt(system_message)

            # 上传文件
            print(f"📤 正在上传文件...")
            try:
                upload_result = platform.upload_files(valid_files)
                if not upload_result:
                    print(f"❌ 文件上传失败")
                    return {
                        "success": False,
                        "stdout": "",
                        "stderr": "文件上传失败",
                    }
                print(f"✅ 文件上传成功")
            except Exception as e:
                print(f"❌ 文件上传失败: {str(e)}")
                return {
                    "success": False,
                    "stdout": "",
                    "stderr": f"文件上传失败: {str(e)}",
                }

            platform.set_suppress_output(False)

            # 构建分析请求
            analysis_request = f"""
请根据以下提示分析这些文件。
{prompt}

请提供详细的分析结果和理由。"""

            # 发送请求并获取分析结果
            print(f"🔍 正在分析文件...")
            analysis_result = platform.chat_until_success(analysis_request)
            print(f"✅ 分析完成")

            # 清理会话
            platform.delete_chat()

            return {"success": True, "stdout": analysis_result, "stderr": ""}

        except Exception as e:
            return {"success": False, "stdout": "", "stderr": f"文件分析失败: {str(e)}"}



# -*- coding: utf-8 -*-
import importlib
import inspect
import os
import sys
from typing import Dict, List, Optional, Type

from jarvis.jarvis_platform.base import BasePlatform
from jarvis.jarvis_utils.config import (
    get_data_dir,
    get_normal_model_name,
    get_normal_platform_name,
    get_thinking_model_name,
    get_thinking_platform_name,
)
from jarvis.jarvis_utils.output import OutputType, PrettyOutput

REQUIRED_METHODS = [
    ("chat", ["message"]),  # 方法名和参数列表
    ("name", []),
    ("delete_chat", []),
    ("set_system_prompt", ["message"]),
    ("set_model_name", ["model_name"]),
    ("get_model_list", []),
    ("upload_files", ["file_list"]),
]


class PlatformRegistry:
    """Platform registry"""

    global_platform_name: str = "yuanbao"
    global_platform_registry: Optional["PlatformRegistry"] = None

    @staticmethod
    def get_platform_dir() -> str:
        user_platform_dir = os.path.join(get_data_dir(), "models")
        if not os.path.exists(user_platform_dir):
            try:
                os.makedirs(user_platform_dir)
                # 创建 __init__.py 使其成为 Python 包
                with open(
                    os.path.join(user_platform_dir, "__init__.py"), "w", errors="ignore"
                ) as f:
                    pass
            except Exception as e:
                PrettyOutput.print(f"创建平台目录失败: {str(e)}", OutputType.ERROR)
                return ""
        return user_platform_dir

    @staticmethod
    def check_platform_implementation(platform_class: Type[BasePlatform]) -> bool:
        """Check if the platform class implements all necessary methods

        Args:
            platform_class: The platform class to check

        Returns:
            bool: Whether all necessary methods are implemented
        """
        missing_methods = []

        for method_name, params in REQUIRED_METHODS:
            if not hasattr(platform_class, method_name):
                missing_methods.append(method_name)
                continue

            method = getattr(platform_class, method_name)
            if not callable(method):
                missing_methods.append(method_name)
                continue

            # 检查方法参数
            import inspect

            sig = inspect.signature(method)
            method_params = [p for p in sig.parameters if p != "self"]
            if len(method_params) != len(params):
                missing_methods.append(f"{method_name}(parameter mismatch)")

        if missing_methods:
            PrettyOutput.print(
                f"平台 {platform_class.__name__} 缺少必要的方法: {', '.join(missing_methods)}",
                OutputType.WARNING,
            )
            return False

        return True

    @staticmethod
    def load_platform_from_dir(directory: str) -> Dict[str, Type[BasePlatform]]:
        """Load platforms from specified directory

        Args:
            directory: Platform directory path

        Returns:
            Dict[str, Type[BasePlatform]]: Platform name to platform class mapping
        """
        platforms: Dict[str, Type[BasePlatform]] = {}

        # 确保目录存在
        if not os.path.exists(directory):
            PrettyOutput.print(f"平台目录不存在: {directory}", OutputType.WARNING)
            return platforms

        # 获取目录的包名
        package_name = None
        if directory == os.path.dirname(__file__):
            package_name = "jarvis.jarvis_platform"

        # 添加目录到Python路径
        if directory not in sys.path:
            sys.path.append(directory)

        # 遍历目录下的所有.py文件
        for filename in os.listdir(directory):
            if filename.endswith(".py") and not filename.startswith("__"):
                module_name = filename[:-3]  # 移除.py后缀
                try:
                    # 导入模块
                    if package_name:
                        module = importlib.import_module(
                            f"{package_name}.{module_name}"
                        )
                    else:
                        module = importlib.import_module(module_name)

                    # 遍历模块中的所有类
                    for _, obj in inspect.getmembers(module):
                        # 检查是否是BasePlatform的子类，但不是BasePlatform本身
                        if (
                            inspect.isclass(obj)
                            and issubclass(obj, BasePlatform)
                            and obj != BasePlatform
                        ):
                            # 检查平台实现
                            if not PlatformRegistry.check_platform_implementation(obj):
                                continue
                            try:
                                # 调用类方法 platform_name
                                platform_name = obj.platform_name()
                                platforms[platform_name] = obj
                            except Exception as e:
                                PrettyOutput.print(
                                    f"实例化或注册平台失败 {obj.__name__}: {str(e)}",
                                    OutputType.ERROR,
                                )
                            break
                except Exception as e:
                    PrettyOutput.print(
                        f"加载平台 {module_name} 失败: {str(e)}", OutputType.ERROR
                    )

        return platforms

    @staticmethod
    def get_global_platform_registry() -> "PlatformRegistry":
        """Get global platform registry"""
        if PlatformRegistry.global_platform_registry is None:
            PlatformRegistry.global_platform_registry = PlatformRegistry()
        return PlatformRegistry.global_platform_registry

    def __init__(self) -> None:
        """Initialize platform registry"""
        self.platforms: Dict[str, Type[BasePlatform]] = {}
        # 从用户平台目录加载额外平台
        platform_dir = PlatformRegistry.get_platform_dir()
        if platform_dir and os.path.exists(platform_dir):
            for (
                platform_name,
                platform_class,
            ) in PlatformRegistry.load_platform_from_dir(platform_dir).items():
                self.register_platform(platform_name, platform_class)
        platform_dir = os.path.dirname(__file__)
        if platform_dir and os.path.exists(platform_dir):
            for (
                platform_name,
                platform_class,
            ) in PlatformRegistry.load_platform_from_dir(platform_dir).items():
                self.register_platform(platform_name, platform_class)

    def get_normal_platform(self) -> BasePlatform:
        platform_name = get_normal_platform_name()
        model_name = get_normal_model_name()
        platform = self.create_platform(platform_name)
        platform.set_model_name(model_name)  # type: ignore
        return platform  # type: ignore

    def get_thinking_platform(self) -> BasePlatform:
        platform_name = get_thinking_platform_name()
        model_name = get_thinking_model_name()
        platform = self.create_platform(platform_name)
        platform.set_model_name(model_name)  # type: ignore
        return platform  # type: ignore

    def register_platform(self, name: str, platform_class: Type[BasePlatform]) -> None:
        """Register platform class

        Args:
            name: Platform name
            model_class: Platform class
        """
        self.platforms[name] = platform_class

    def create_platform(self, name: str) -> Optional[BasePlatform]:
        """Create platform instance"""
        """Create platform instance

        Args:
            name: Platform name

        Returns:
            BasePlatform: Platform instance
        """
        if name not in self.platforms:
            PrettyOutput.print(f"未找到平台: {name}", OutputType.WARNING)
            return None

        try:

            platform = self.platforms[name]()
            return platform
        except Exception as e:
            PrettyOutput.print(f"创建平台失败: {str(e)}", OutputType.ERROR)
            return None

    def get_available_platforms(self) -> List[str]:
        """Get available platform list"""
        return list(self.platforms.keys())
